options linesize=80 pagesize=66 nocenter;

/* Note that the 1988 SIPP data has space for 32 months of   */
/* data, but only the first 24 month are used.               */

/* Uses new method of assiging STATE for each month of data. */
/* SIPP data has 32 months of data for each person but only  */
/* 8 values for state - each 4-month period maps to 1 state. */
/* Rather than use a state for each of the 4 corresponding   */
/* months in the data, this program matches the HHADDI values*/
/* with that corresponding state for all people within one   */
/* PPID value, then creates a table of HHADDI/STATE with the */
/* most frequently occurring STATE value for each HHADDI.    */
/* The resulting table is then used to define the STATE for  */
/* each occurrance of that HHADDI.  This enables correction  */
/* of the state value for situations such as when a person   */
/* lives at two different HHADDIs (each in a different state)*/
/* within one 4-month period.                                */

/* Check whether the PNPT and ENTPT for each child is        */
/* consistent with the "reference person" and "child of      */
/* reference person" indicators.                             */
/* Also checks if RELAT 4 or 5 person has ENTPT/PNPT that    */
/* shows reference person (or spouse) is their parent.       */

/* Definitions:                                              */
/*  Child         a person less than or equal to 18 years old*/
/*  KID           a "child of reference person" (RRP=4 for a */
/*                primary family or FAMREL=3 for a subfamily)*/
/*                and it does not include other children in  */
/*                the household (either related or unrelated)*/
/*  SingleMother  a female "reference person" with no spouse */
/*                present and at least one KID present       */
/*                during the month, with a MaritalStatus of  */
/*                3(widow), 4(divorced), 5(separated), or 6  */
/*                (never married).                           */

/* Program creates these new variables:                      */
/*                                                           */
/*  SMOM        0 = not a Single Mother in this month        */
/*              1 = a Single Mother in this month            */
/*                                                           */
/*  NUMMTHS     0 = not a Single Mother in any months        */
/*              n = total number of months as a Single Mother*/
/*                                                           */
/*  NMTOT       total number of months with PPMIS=1          */
/*                                                           */
/*  RELAT       1 = reference person (living with relatives) */
/*              2 = spouse of reference person               */
/*              3 = child of reference person                */
/*              4 = other relative of reference person       */
/*             14 = other relative of reference person, less */
/*                  than or equal to 18 years old, who has   */
/*                  reference person as their Guardian       */
/*              5 = "unrelated individual" (could be related */
/*                  to others in household) - corresponds to */
/*                  three categories of SIPP RRP variable:   */
/*                  (2) "reference person alone or with      */
/*                  non-relatives"; (6) "non-relative of     */
/*                  reference person but related to others   */
/*                  in household"; (7) "not related to anyone*/
/*                  else in household (secondary individual)"*/
/*                                                           */
/*  FAMSTR   (Family structure includes people with PPMIS=2) */
/*              0 = individuals living alone                 */
/*              1 = single mom with kids                     */
/*              2 = two parents with kids                    */
/*              3 = single dad with kids                     */
/*              4 = female head, no kids                     */
/*              5 = dual head, no kids                       */
/*              6 = male head, no kids                       */
/*              7 = no adult head, but with kids             */
/*                                                           */
/*  MOMPRES     0 = no female ref person or spouse this month*/
/*              1 = female ref person or spouse present      */
/*              2 = female ref person but with PPMIS=2       */
/*  DADPRES     0 = no male ref person or spouse this month  */
/*              1 = male ref person or spouse present        */
/*              2 = male ref person but with PPMIS=2         */
/*                                                           */
/*  NUMKIDS     = total number of KIDS in family this month  */
/*              (i.e., RELAT=3 children)                     */
/*                                                           */
/*  OCHREFGD    = total number of RELAT=14 children present  */
/*              this month (i.e., other relative of RefPerson*/
/*              who has RefPerson as their Guardian)         */
/*                                                           */
/* Note: In all the following variables, "kid" includes both */
/* RELAT=3 children (child of reference person) and RELAT=14 */
/* children (other relative of reference person, who has the */
/* RefPerson as their Guardian                               */ 
/*  KIDLT18     = number of kids less than 18                */
/*  KIDLT12     = number of kids less than 12                */
/*  KIDLT6      = number of kids less than  6                */
/*  KIDLT2      = number of kids less than  2                */
/*  OLDEST      = age of oldest kid in family this month     */
/*  YOUNGEST    = age of youngest kid in family this month   */
/*                                                           */
/*                                                           */
/*  OTHCHLDN    = number of other children present this month*/
/*              in addition to those with RELAT=3,14         */
/*              which includes people less than or equal to  */
/*              18 years old with RELAT=4 (other relative of */
/*              reference person) or RELAT=5 (unrelated)     */
/*                                                           */
/*  CHPRES      = total number of related children present   */
/*              this month, where "related child" includes   */
/*              KID as defined above and also includes people*/
/*              less than or equal to 18 years with RELAT=4  */
/*              or =14 (other relative of reference person)  */
/*                                                           */
/*  FAMID       = family number for PPID/HHADDI in a month   */
/*  NPERSNS     = number of people in family this month      */
/*                                                           */


/* Macro to expand sequence of 9 to sequence of 24 */
%macro x6to24(name);
 %do k=24 %to 1 %by -1;
   %let j = %eval((&k + 3)/4);
  &name&k = &name&j;
 %end;
%mend x6to24;


filename in1 PIPE 'gunzip -c /files2/ldshore/sipp/1988/da6028.gz'; 
/* filename in1 '/export/files7/welfare/sep04_a/da6028; */

/* Read some variables from the 1988 SIPP data */
libname x '';

data sipp88;

 infile in1 lrecl=12601 recfm=f missover; 

 input 
@7	ROT		1.
@8	PPID		9.
@17	PPENTRY		2.
@19	PPPNUM 		3.
@26	(HHADDI1-HHADDI24)	(2.)
@98	(PPMIS1-PPMIS24)	(1.)
@1236	SEX		1.
@1240	(RRP1-RRP24)	(1.)
@1272	(AGE1-AGE24)	(2.)
@1336	(MS1-MS24)	(1.)
@1368	(FAMTYP1-FAMTYP24)	(1.)
@1400	(FAMREL1-FAMREL24)	(1.)
@1432	(FAMNUM1-FAMNUM24)	(1.)
@1464	(PNSP1-PNSP24)	(3.)
@1560	(ENTSP1-ENTSP24)	(2.)
@1624	(PNPT1-PNPT24)	(3.)
@1720	(ENTPT1-ENTPT24)	(2.)
@1848   UBRTHMN         2.
@1850   UBRTHYR         4.
@1854   (pngd1-pngd6)           (3.)
@1878   (entgd1-entgd6)         (2.)
@2190	(GEOSTE1-GEOSTE6)	(2.)
;
%x6to24(geoste);
%x6to24(pngd);
%x6to24(entgd);
run;

/* Sort in order by PPID,PPENTRY,PPPNUM */
proc sort data=sipp88; by ppid ppentry pppnum;

/* options obs=34;  */

/* Look at all records for a PPID and build a matrix of all it's people */
data x.fam88;
 set sipp88 end=end_file; by ppid;

/* Fix values in the raw data that seem to be data errors */
/*                                                        */
/* Baby lives in subfamily with mother and older sister;  */
/* PNPT of baby is correct for all months but ENTPT wrong */
/* for four months - fix this.                            */
 if ppid eq 460123008 and ppentry eq 41 and pppnum eq 401 then
   do; entpt17=11; entpt18=11; entpt19=11; entpt20=11; end;
/* Six year old boy lives in subfamily with mother; PNPT  */
/* and ENTPT show she is mother for all months, but she   */
/* has zeros for FAMTYP,FAMREL,FAMNUM for 7 months - fix. */
 if ppid eq 634369925 and ppentry eq 11 and pppnum eq 103 then
   do; famtyp17=3; famtyp18=3; famtyp19=3; famtyp20=3;
       famtyp21=3; famtyp22=3; famtyp23=3;
       famrel17=1; famrel18=1; famrel19=1; famrel20=1;
       famrel21=1; famrel22=1; famrel23=1;
       famnum17=1; famnum18=1; famnum19=1; famnum20=1;
       famnum21=1; famnum22=1; famnum23=1; end;
/*                                                        */
/* Define arrays for the data read for a person */
 array xhhaddi{24} hhaddi1-hhaddi24;
 array xppmis{24} ppmis1-ppmis24 ;
 array xrrp{24}  rrp1-rrp24;
 array xage{24} age1-age24 ;
 array xms{24} ms1-ms24;
 array xfamtyp{24} famtyp1-famtyp24;
 array xfamrel{24} famrel1-famrel24;
 array xfamnum{24} famnum1-famnum24;
 array pnsp{24} ;
 array entsp{24} ;
 array pnpt{24} ;
 array entpt{24} ;
 array pngd{24} ;
 array entgd{24} ;
 array geoste{24} ;

/* Define a series of temporary arrays to hold the data */
/* There are 24 months in the 1988 SIPP data */
/* Allow for a maximum of 34 people in each PPID group */
 array tentry{34} _temporary_ ;
 array tnum{34} _temporary_  ;
 array tsex{34} _temporary_ ;
 array tbrmn{34} _temporary_ ;
 array tbryr{34} _temporary_ ;
 array thhad{24,34} _temporary_ ;
 array tpmis{24,34} _temporary_ ;
 array trrp{24,34} _temporary_ ;
 array tage{24,34} _temporary_ ;
 array tms{24,34} _temporary_ ;
 array tfmtp{24,34} _temporary_ ;
 array tfmrl{24,34} _temporary_ ;
 array tfmnm{24,34} _temporary_ ;
 array tpnsp{24,34} _temporary_ ;
 array tents{24,34} _temporary_ ;
 array tpnpt{24,34} _temporary_ ;
 array tentp{24,34} _temporary_ ;
 array tpngd{24,34} _temporary_ ;
 array tengd{24,34} _temporary_ ;
 array tfid{24,34} _temporary_ ;
 array tfid2{24,34} _temporary_ ;
 array tsid{24,34} _temporary_ ;
 array tgeos{24,34} _temporary_ ;

/* NP is the person number of someone in this PPID group */
 retain np;
 if first.ppid then np=1; else np=np+1;

/* Put person's identifying info into temporary array */
 tentry(np)=ppentry;
 tnum(np)=pppnum;
 tsex(np)=sex;
 tbrmn(np)=ubrthmn;
 tbryr(np)=ubrthyr;

/* Create actual dates (YYMM) for each of the 24 months */
 array xyear(24) _temporary_ ;
 do i=1 to 24;
  xyear{i}=0;
 end;
 if rot=1 then xyear(1)=88;
  else if rot=2 then xyear(1)=87;
  else if rot=3 then xyear(1)=87;
  else if rot=4 then xyear(1)=87;
  else xyear(1)=.;
 array xmonth(24) _temporary_ ;
 do i=1 to 24;
  xmonth{i}=0;
 end;
 if rot=1 then xmonth(1)=01;
  else if rot=2 then xmonth(1)=10;
  else if rot=3 then xmonth(1)=11;
  else if rot=4 then xmonth(1)=12;
  else xmonth(1)=.;
 do i=1 to 23;
  if xmonth{i} lt 12 then
    do;
    xmonth{i+1}=xmonth{i}+1;
    xyear{i+1}=xyear{i};
    end;
  else if xmonth{i}=12 then
    do;
    xmonth{i+1}=01;
    xyear{i+1}=xyear{i}+1;
    end;
 end;
 array xdate(24) date1-date24;
 do i=1 to 24;
  xdate{i}=(xyear{i}*100) + (xmonth{i});
 end;

/* Loop for all 24 months and put person data into temporary array*/
 do mo=1 to 24; 
  thhad(mo,np)=xhhaddi(mo);
  tpmis(mo,np)=xppmis(mo);
  trrp(mo,np)=xrrp(mo);
  tage(mo,np)=xage(mo);
  tms(mo,np)=xms(mo);
  tfmtp(mo,np)=xfamtyp(mo);
  tfmrl(mo,np)=xfamrel(mo);
  tfmnm(mo,np)=xfamnum(mo);
  tpnsp(mo,np)=pnsp(mo);
  tents(mo,np)=entsp(mo);
  tpnpt(mo,np)=pnpt(mo);
  tentp(mo,np)=entpt(mo);
  tpngd(mo,np)=pngd(mo);
  tengd(mo,np)=entgd(mo);
  tgeos(mo,np)=geoste(mo);
/* Zero out HHADDI if PPMIS ne 1 OR 2 */
  if tpmis(mo,np) not in(1,2) then thhad(mo,np)=0;
 end;
/* Zero STATE if all 4 months of interview period have PPMIS ne 1 */
/* (need at least one valid month for STATE to be kept in data).  */
 do mo4=1 to 24 by 4;
  if tpmis(mo4,np) ne 1 and tpmis(mo4+1,np) ne 1
    and tpmis(mo4+2,np) ne 1 and tpmis(mo4+3,np) ne 1 then
    do;
     tgeos(mo4,np)=0;
     tgeos(mo4+1,np)=0;
     tgeos(mo4+2,np)=0;
     tgeos(mo4+3,np)=0;
    end;
 end;



/* Do special things when reach last person in the PPID group  */
 if last.ppid then
 do;
/* Compute new "Family Variables" */
/* First, define FID2 and SID as documented in SIPP manual */
/* Note: NP = total number of people in this PPID group */
 do ip=1 to np;		/* Loop for each person */
 do mo=1 to 24;		/* Loop for each month  */
  if tpmis(mo,ip) in (1,2) then		/* interview status = 1 OR 2!! */
  do;
   if tfmtp(mo,ip) eq 0 then		/* primary family */
     do;
     tfid(mo,ip)=1;
     tfid2(mo,ip)=1;
     tsid(mo,ip)=0;
     end;
   else if tfmtp(mo,ip) eq 1 then	/* secondary individual */
     do;
     tfid(mo,ip)=10000+ip;
     tfid2(mo,ip)=10000+ip;
     tsid(mo,ip)=0;
     end;
   else if tfmtp(mo,ip) eq 2 then	/* unrelated subfamily */
     do;
     tfid(mo,ip)=100+tfmnm(mo,ip);
     tfid2(mo,ip)=100+tfmnm(mo,ip);
     tsid(mo,ip)=0;
     end;
   else if tfmtp(mo,ip) eq 3 then	/* related subfamily */
     do;
     tfid(mo,ip)=1;
     tfid2(mo,ip)=0;
     tsid(mo,ip)=20000+tfmnm(mo,ip);
     end;
   else if tfmtp(mo,ip) eq 4 then	/* primary individual */
     do;
     tfid(mo,ip)=10000+ip;
     tfid2(mo,ip)=10000+ip;
     tsid(mo,ip)=0;
     end;
  end;  /* end of PP-MIS = 1 OR 2 block (tpmis(mo,ip) in (1,2)) */
 end;   /* end of Month loop (mo=1 to 24) */
 end;   /* end of People loop (ip=1 to np) */


/* FAMILY DEFINITION SECTION *****************************************/
/* For each month, identify who makes up a family */
 do mo=1 to 24;  /* Loop for each month */
/* Create array FAMLY - each column contains the person number  */
/* of all the members of one family in this PPID group.         */
/* Variable NUMFAMLY = the number of families in the PPID group */
/* Array NPERSN contains the number of people in each family    */
 array fmly{34,34};  /* could be 34 1-person families or 1 34-person family */
 array npersn(34);     /* count of number of persons in each family */
 do i=1 to 34;
 npersn(i)=0;
 do j=1 to 34;
 fmly(i,j)=0;
 end;
 end;
/* Keep track of maximum number of families per PPID */
/* and maximum number of people per family.          */
 retain maxfam maxfamid maxfammo maxppl maxpplid maxpplmo;
 if _n_ eq 1 then do;maxfam=1;maxfamid=ppid;maxfammo=1;
                     maxppl=1;maxpplid=ppid;maxpplmo=1;end;
/* Put first person into "family #1" */
 numfamly=1;
 npersn(1)=0;
 do ip=1 to np while (fmly(1,1) eq 0);
  if ((tsid(mo,ip) eq 0 and tfid2(mo,ip) ne 0)        or
     (tsid(mo,ip) not in(0,.) and tfid2(mo,ip) eq 0)) 
     and tpmis(mo,ip) in (1,2) then  
    do;
    npersn(1)=1;
    fmly(numfamly,npersn(1))=ip;
    end;
 end;
/* Are there any people at all in this month? */
 if npersn(1) eq 0 then goto nobody;
/* If someone, check to be sure that 1st person has an address */
 prsn1= fmly(numfamly,npersn(1));
 if  tpmis(mo,prsn1) not in(1,2) or thhad(mo,prsn1) in (0,.) then
  do;
  put 'ERROR: ' ppid tentry(prsn1) tnum(prsn1) ' Person #1 has no address!';
  put 'month ' mo 'sid,fid2= ' tsid(mo,prsn1) tfid2(mo,prsn1);
  put 'ppmis= ' tpmis(mo,prsn1) ' hhaddi= ' thhad(mo,prsn1);
  end;

/* Check all the other people in that month */
 do ip=(prsn1+1) to np;
 if tpmis(mo,ip) in(1,2) then   /* only check people-months when PP-MIS = 1,2 */
 do;
    match1=0;
    match2=0;
/* Is this person (not in related subfamily) in a family we've already seen? */
  if tsid(mo,ip) eq 0 and tfid2(mo,ip) ne 0 then
    do i=1 to numfamly;
      if tfid2(mo,ip) eq tfid2(mo,fmly(i,1)) and
         thhad(mo,ip) eq thhad(mo,fmly(i,1)) then match1=i;
    end;
/* Is this person in a related subfamily that we've already seen? */
  if tsid(mo,ip) not in(0,.) then
    do i=1 to numfamly;
      if tsid(mo,ip)  eq tsid(mo,fmly(i,1))  and
         thhad(mo,ip) eq thhad(mo,fmly(i,1)) then match2=i;
    end;
/* If these was a match, add this person to existing family */
  if match1 ne 0 then	/* Match was NOT in a related subfamily */
    do;
    npersn(match1)=npersn(match1)+1;	/* Increment # people in family */
    /* Keep track of maximum number of people in any family */
    if  npersn(match1) gt maxppl then do;
        maxppl=npersn(match1); maxpplid=ppid; maxpplmo=mo; end;
    fmly(match1,npersn(match1))=ip;	/* Put person # in family array */
    end;
  if match2 ne 0 then	/* Match WAS in a related subfamily */
    do;
    npersn(match2)=npersn(match2)+1;	/* Increment # people in family */
    /* Keep track of maximum number of people in any family */
    if  npersn(match2) gt maxppl then do;
        maxppl=npersn(match2); maxpplid=ppid; maxpplmo=mo; end;
    fmly(match2,npersn(match2))=ip;	/* Put person # in family array */
    end;
/* If there was not a match, start another family */
  if match1 eq 0 and match2 eq 0 then
    do;
    numfamly=numfamly+1;
    npersn(numfamly)=1;
    fmly(numfamly,1)=ip;
/* Keep track of maximum number of families in any PPID group */
    if numfamly gt maxfam then do;
       maxfam=numfamly; maxfamid=ppid; maxfammo=mo; end;
    end;
 end;  /* end of PP-MIS=1 OR 2 block (tpmis(mo,ip) in(1,2)) */
 end;  /* end of People loop (ip=(prsn1+1) to np) */

/* Check that all the people in each "family" are living */
/* together at the same address that month.              */
 do i=1 to numfamly;
 sameadd=1;
 if npersn(i) gt 1 then
  do j=2 to npersn(i);
   if thhad(mo,fmly(i,j)) ne thhad(mo,fmly(i,j-1)) then sameadd=0;
  end;
 if sameadd ne 1 then put 'ERROR: Family has different addresses.'
   ' Month=' mo '  PPID=' ppid '  FID2/SID=' fmly(i,1);
 end;  /* end of Family loop (i=1 to numfamly) */

/* Loop through each person in "family" and identify their role  */
/* (1=head,2=spouse,3=kid,4=other rel,5=unrelated individual)  */
 do i=1 to numfamly;
 do j=1 to npersn(i);
 array relate{24,34} _temporary_ ;
/* Subfamily */
 if tsid(mo,fmly(i,j)) not in (0,.) 	/* Related subfamily */
   or tfmtp(mo,fmly(i,j)) eq 2		/* Unrelated subfamily */
   then relate(mo,fmly(i,j))=tfmrl(mo,fmly(i,j));
/* Not a Subfamily */
 if tsid(mo,fmly(i,j)) eq 0 and tfmtp(mo,fmly(i,j)) ne 2 then
   do;
   if trrp(mo,fmly(i,j)) eq 1 then 
     relate(mo,fmly(i,j))=1;  /* reference person */
   if trrp(mo,fmly(i,j)) eq 3 then
     relate(mo,fmly(i,j))=2;  /* spouse */
   if trrp(mo,fmly(i,j)) eq 4 then
     relate(mo,fmly(i,j))=3;  /* kid */
   if trrp(mo,fmly(i,j)) eq 5 then
     relate(mo,fmly(i,j))=4;  /* other relative */
   if trrp(mo,fmly(i,j)) in (2,6,7) then
     relate(mo,fmly(i,j))=5;  /* unrelated individual */
   /* Note RELAT=5 could be related to others in household */
   end; 
 end;  /* end of People in Family loop (j=1 to npersn(i)) */
 end;  /* end of Families loop (i=1 to numfamly) */

/* Loop through each person in "family"; set flags, count kids */
 do i=1 to numfamly;
/* Check Entry_addesss of Parent & Person_number of Parent    */
/* (ENTPT & PNPT) to see if (1) each KID's matches the ref    */
/* person (or spouse) in this month and (2) any OtherRelative */
/* or NonRelative matches the ref person or spouse.           */
/* (Need separate loop: parent might be after child in order) */
 file 'SIPP_families88.conflicting_parentinfo';
/* First find family reference person and/or spouse */
 rpnpt=.; rentpt=.; spnpt=.; sentpt=.; /* initialize for family */
 do j=1 to npersn(i);
 ip=fmly(i,j); /* person number of person in this family */
 if relate(mo,ip) eq 1 then  /* if it's the reference person, then */
   do;
   rpnpt=tnum(ip);         /* store their PPPNUM and */
   rentpt=tentry(ip);      /* store their PPENTRY    */
   end;
 if relate(mo,ip) eq 2 then  /* if it's the spouse of ref person, then */
   do;
   spnpt=tnum(ip);         /* store their PPPNUM and */
   sentpt=tentry(ip);      /* store their PPENTRY    */
   end;
 end;
/* Next check kids */
 do j=1 to npersn(i);
 ip=fmly(i,j);	/* person number of person in this family */
 if relate(mo,ip) eq 3 	/* child of ref person (i.e, KID) */
   and tage(mo,ip) le 18 /* child not older than 18 */
   and tpnpt(mo,ip) not in (0,999) /* there is person# of parent */
  then	/* check if that person# is RefPerson/Spouse this month */
  do;
   pmatch=0;	/* flag set to "no match" with parent Entry/Pnum */
   if tentp(mo,ip) ne 0  /* entry # of parent not missing */
     and ((tentp(mo,ip) eq rentpt and tpnpt(mo,ip) eq rpnpt) /* match ref */
     or (tentp(mo,ip) eq sentpt and tpnpt(mo,ip) eq spnpt))  /* match spouse */
      then pmatch=1; /* ENTPT/PNPT is a valid match */
   if tentp(mo,ip) eq 0 then /* entry # of parent is missing */
     do; 
      if tpnpt(mo,ip) eq rpnpt then /* PNPT matches reference person */
         do;
         tentp(mo,ip)=rentpt; /* set missing ENTPT to reference person */
         pmatch=1;
         end;
      if tpnpt(mo,ip) eq spnpt then /* PNPT matches spouse of ref person */
         do;
         tentp(mo,ip)=sentpt; /* set missing ENTPT to spouse of ref prsn */
         pmatch=1;
         end;
     end;
   if pmatch eq 0 then 
     do;
     put "KID's ENTPT/PNPT does not match RefPerson or Spouse this month";
     put 'PPID=' ppid ' PPENTRY=' tentry(ip) ' PPPNUM=' tnum(ip)
      'Month=' mo ' ENTPT=' tentp(mo,ip) ' PNPT=' tpnpt(mo,ip);
     tentp(mo,ip)=0; /* reset ENTPT to 0 */
     tpnpt(mo,ip)=0; /* reset PNPT to 0 */
     end;
  end;  /* end of KID in Family block (relate(mo,ip) eq 3) */
/* Also check if person is "other relative or nonrelative" but */
/* has parent information saying the ref person is their parent*/
 if relate(mo,ip) in (4,5)  /* other rel or non rel of ref person */
   and tage(mo,ip) le 18 /* child not older than 18 */
   and tpnpt(mo,ip) not in (0,999) /* there is person# of parent */
  then  /* check if that person# is RefPerson/Spouse this month */
  do;
   pmatch=0;    /* flag set to "no match" with parent Entry/Pnum */
   /* Loop through all people in family this month */
   do k=1 to npersn(i);
     if tpnpt(mo,ip) eq tnum(fmly(i,k)) and /* person# matches */
      relate(mo,fmly(i,k)) in (1,2) then    /* ref person or spouse */
      do;
      if tentp(mo,ip) ne 0 and               /* Entry# of parent not missing */
         tentp(mo,ip) eq tentry(fmly(i,k))  /* entry# matches */
      then pmatch=1;
      if tentp(mo,ip) eq 0 and               /* Entry# of parent IS missing */
       (tentry(ip) eq tentry(fmly(i,k)) or  /* kid's entry = parent entry */
        thhad(mo,ip) eq tentry(fmly(i,k))) /* kids's HHADDI = parent entry */
      then pmatch=1;
      end;
   end;
   if pmatch eq 1 then
     do;
     put "OtherRelative or Unrelated ENTPT/PNPT matches ref person or spouse";
     put 'PPID=' ppid ' PPENTRY=' tentry(ip) ' PPPNUM=' tnum(ip)
      'Month=' mo ' ENTPT=' tentp(mo,ip) ' PNPT=' tpnpt(mo,ip);
     end;
  end;  /* end of Other/Unrelat block (relate(mo,ip) in (4,5) ) */
/* Finally, check if person is "other relative or nonrelative",  */
/* has NO parent information, and has Guardian information saying */
/* the RefPerson or Spouse of RefPerson is their Guardian         */
 if relate(mo,ip) in (4,5)  /* other rel or non rel of ref person */
   and tage(mo,ip) le 18 /* child not older than 18 */
   and tpnpt(mo,ip) in (0,999) /* there is NO person# of parent */
   /* and the RefPerson/Spouse is designated as the child's Guardian */
   and ((tpngd(mo,ip) eq rpnpt and tengd(mo,ip) eq rentpt)   
     or (tpngd(mo,ip) eq spnpt and tengd(mo,ip) eq sentpt))
  then  /* mark this child as having that RefPerson/Spouse as Guardian */
   relate(mo,ip)=relate(mo,ip)+10; /* recode RELAT to 14 */
 end;  /* end of People in Family loop (j=1 to npersn(i)) */

/* If there is no parent defined for a "Child of RefPerson" via     */
/* ENTPT/PNPT, then define reference person (RELAT=1) as the parent */
 do j=1 to npersn(i);
 ip=fmly(i,j); /* person number of person in  this family */
 if relate(mo,ip) eq 3  /* child of ref person (i.e, KID) */
   and tage(mo,ip) le 18 /* child not older than 18 */
   and (tpnpt(mo,ip) in (0,999)  /* there is NO person# of parent */
     or tentp(mo,ip) in (0,999)) /* there is NO entry# of parent */
  then  
  do;
  tpnpt(mo,ip)=rpnpt;
  tentp(mo,ip)=rentpt;
  end;
 end;

/* Initialize variables */
 mompres=0;
 dadpres=0;
 chpres=0;
 numkids=0;
 othchldn=0;
 ochrefgd=0;
 kidlt18=0;
 kidlt12=0;
 kidlt6=0;
 kidlt2=0;
 oldest=.;
 youngest=.;
 do j=1 to npersn(i);
  if relate(mo,fmly(i,j)) in (1,2) and tsex(fmly(i,j)) eq 1
    and tpmis(mo,fmly(i,j)) eq 1 then dadpres=1;
  if relate(mo,fmly(i,j)) in (1,2) and tsex(fmly(i,j)) eq 1
    and tpmis(mo,fmly(i,j)) eq 2 then dadpres=2;
  if relate(mo,fmly(i,j)) in (1,2) and tsex(fmly(i,j)) eq 2
    and tpmis(mo,fmly(i,j)) eq 1 then mompres=1;
  if relate(mo,fmly(i,j)) in (1,2) and tsex(fmly(i,j)) eq 2
    and tpmis(mo,fmly(i,j)) eq 2 then mompres=2;
  if relate(mo,fmly(i,j)) eq 3 and tage(mo,fmly(i,j)) le 18 then
    numkids=numkids+1; /* Number of RELAT=3 children in family */
  if relate(mo,fmly(i,j)) eq 14 and tage(mo,fmly(i,j)) le 18 then
    ochrefgd=ochrefgd+1; /* Number of RELAT=14 children in family */
/* treat RELAT=14 children just like REAL KIDS when summing them */
  if relate(mo,fmly(i,j)) in (3,14)   
   and tage(mo,fmly(i,j)) le 18 then     /* KIDs block */
    do;
    if tage(mo,fmly(i,j)) lt 18 then kidlt18=kidlt18+1;
    if tage(mo,fmly(i,j)) lt 12 then kidlt12=kidlt12+1;
    if tage(mo,fmly(i,j)) lt 6 then kidlt6=kidlt6+1;
    if tage(mo,fmly(i,j)) lt 2 then kidlt2=kidlt2+1;
    if youngest eq . or tage(mo,fmly(i,j)) lt youngest then
       youngest=tage(mo,fmly(i,j));
    if oldest eq . or tage(mo,fmly(i,j)) gt oldest then
       oldest=tage(mo,fmly(i,j));
    end;  /* end of KIDS block */
  if relate(mo,fmly(i,j)) in (4,5) and tage(mo,fmly(i,j)) le 18 then
    othchldn=othchldn+1; /* Additional children (RELAT=4,5) in family */
  if relate(mo,fmly(i,j)) in (3,4,14) and tage(mo,fmly(i,j)) le 18 then
    chpres=chpres+1;  /* all children in family related to RefPerson */
 end;  /* end of People in Family loop (j=1 to npersn(i)) */

/* Define the family structure */
 famstr=0;			/* individuals living alone */
 if mompres ne 0 and dadpres eq 0 and (numkids+ochrefgd) gt 0 
    then famstr=1;		/* single mom with kids */
 if mompres ne 0 and dadpres ne 0 and (numkids+ochrefgd) gt 0
    then famstr=2;		/* two parents with kids */
 if mompres eq 0 and dadpres ne 0 and (numkids+ochrefgd) gt 0
    then famstr=3;		/* single dad with kids */
 if mompres ne 0 and dadpres eq 0 and (numkids+ochrefgd) eq 0 
    then famstr=4;		/* female head, no kids */
 if mompres ne 0 and dadpres ne 0 and (numkids+ochrefgd) eq 0 
    then famstr=5;		/* dual head, no kids */
 if mompres eq 0 and dadpres ne 0 and (numkids+ochrefgd) eq 0 
    then famstr=6;		/* male head, no kids */
 if mompres eq 0 and dadpres eq 0 and (numkids+ochrefgd) gt 0 
    then famstr=7;		/* no adult head, but with kids */
/* Store the family information in each person's data */
 array momp{24,34} _temporary_ ;
 array dadp{24,34} _temporary_ ;
 array chp{24,34} _temporary_ ;
 array othc{24,34} _temporary_ ;
 array nkid{24,34} _temporary_ ;
 array orfg{24,34} _temporary_ ;
 array k18{24,34} _temporary_ ;
 array k12{24,34} _temporary_ ;
 array k6{24,34} _temporary_ ;
 array k2{24,34} _temporary_ ;
 array old{24,34} _temporary_ ;
 array yng{24,34} _temporary_ ;
 array fstr{24,34} _temporary_ ;
 array sm{24,34} _temporary_ ;
 array fmid{24,34} _temporary_ ;
 array npsn{24,34} _temporary_ ;
 do j=1 to npersn(i);
  momp(mo,fmly(i,j))=mompres;
  dadp(mo,fmly(i,j))=dadpres;
  chp(mo,fmly(i,j))=chpres;
  othc(mo,fmly(i,j))=othchldn;
  nkid(mo,fmly(i,j))=numkids;
  orfg(mo,fmly(i,j))=ochrefgd;
  k18(mo,fmly(i,j))=kidlt18;
  k12(mo,fmly(i,j))=kidlt12;
  k6(mo,fmly(i,j))=kidlt6;
  k2(mo,fmly(i,j))=kidlt2;
  old(mo,fmly(i,j))=oldest;
  yng(mo,fmly(i,j))=youngest;
  fstr(mo,fmly(i,j))=famstr;
  fmid(mo,fmly(i,j))=i;
  npsn(mo,fmly(i,j))=npersn(i);
  ip=fmly(i,j); /* person number of person in this family */
  sm(mo,ip)=0;   /* Not a SingleMother this month */
  if fstr(mo,ip) eq 1           /* Family Structure "Single mom with kids" */
    and tpmis(mo,ip) eq 1       /* PPMIS = 1 for that month */
    and tsex(ip) eq 2           /* female */
    and relate(mo,ip) in (1,2)  /* either reference person or spouse of ref */
    and tms(mo,ip) in (3,4,5,6) /* only MS widow,divorce,separated,nevermar */
    then sm(mo,ip)=1;           /* Person IS a SingleMother this month */
 end;  /* end of People in Family loop (j=1 to npersn(i)) */
 end;  /* end of Families loop (i=1 to numfamly) */
nobody: ; /* jump here if cannot put even 1 person into family#1 this month */
 end;  /* end of Month loop (mo=1 to 24) for who makes up a family */


/* STATE DEFINITION SECTION ******************************************/
/* Define STATE based on information from all people in PPID */

/* Make list of all HHADDIs for all people in this PPID */
/* (allow for up to 50) */
 array lhh{50} _temporary_ ;
/* Zero out array */
 do i=1 to 50;
  lhh(i)=0;
 end;
 nlhh=0; /* number of different HHADDIs in this PPID */
 do ip=1 to np; /* NP = total number of people in this PPID group */
 do mo=1 to 24;
  inlist=0;
  do i=1 to nlhh; /* Loop through all HHADDIs in the list already */
   if thhad(mo,ip) ne 0 and thhad(mo,ip) eq lhh(i) then inlist=1;
  end;
/* If non-zero HHADDI isn't in the list, add it */
  if inlist eq 0 and thhad(mo,ip) ne 0 then
  do;
   nlhh=nlhh+1;
   lhh(nlhh)=thhad(mo,ip);
  end;
 end;  /* end of month loop */
 end;  /* end of people loop */
/* Be sure total is within limit of 50 allowed by array */
 if nlhh gt 50 then put 'ERROR: TOO MANY HHADDI ' ppid nlhh;

/* Make array of states for each HHADDI */
/* (allow up to 10) */
 array lstat{50,10} _temporary_ ;  /* STATE numbers for each HHADDI */
 array fstat{50,10} _temporary_ ;  /* freq of STATE numbers for each HHADDI */
 array nstat{50} _temporary_ ;  /* number of states for each HHADDI */
/* Zero out arrays */
 do i=1 to 50;
  nstat(i)=0;
 do j=1 to 10;
  lstat(i,j)=0;
  fstat(i,j)=0;
 end;
 end;
/* Store STATE number matches, plus frequency of matches */
 do ip=1 to np; /* NP = total number of people in this PPID group */
 do mo4=1 to 24 by 4; /* month loop */
/* First, get one HHADDI number */
  hhx=0; /* HHX is a HHADDI number */
  do j=0 to 3; /* interview period (4 month) loop */
  if thhad(mo4+j,ip) ne 0 then
    if hhx eq 0 then hhx=thhad(mo4+j,ip);
     else if hhx ne thhad(mo4+j,ip) then hhx=999; /* 999 is mismatch */
  end;  /* end of interview period (4 month) loop */
/* If all HHADDIs in 4-month period match and are not 0, and if */
/* STATE is not 0, find HHADDI in list and update the STATE &   */
/* frequency values for it.                                     */
  if hhx not in(0,999) and tgeos(mo4,ip) ne 0 then
   do;
   hnum=0;  /* HNUM = number of the HHADDI in the list */
   do i=1 to nlhh; /* loop through all HHADDIs */
    if hhx eq lhh(i) then hnum=i;
   end;
   if hnum eq 0 then put 'ERROR: HHADDI not found in list';
/* If this STATE number not already stored, store it now */
/* Search for this state in the array for this HHADDI */
   instate=0;
   do i=1 to nstat(hnum);
    if lstat(hnum,i) eq tgeos(mo4,ip) then
     do;
     instate=1; /* found state in array, update frequency */
     fstat(hnum,i)=fstat(hnum,i)+1;
     end;
   end;
   if instate eq 0 then /* state not found, add it */
     do;
     nstat(hnum)=nstat(hnum)+1;
     lstat(hnum,nstat(hnum))=tgeos(mo4,ip); /* store the state */
     fstat(hnum,nstat(hnum))=fstat(hnum,nstat(hnum))+1; /* frequency */
     end;
    end;
 end;  /* end of month loop */
 end;  /* end of people loop */

/* Build final table of HHADDI/STATE matches */
 do i=1 to nlhh;
/* Check for HHADDIs with no state                                */
/* (occurs when no interview period had this HHADDI all 4 months) */
 if nstat(i) eq 0 then
/* Find all states shown for the HHADDI */ 
  do ip=1 to np;
  do mo=1 to 24;
/* When find HHADDI with non-zero state, put in table */
   if lhh(i) eq thhad(mo,ip) and tgeos(mo,ip) ne 0 then
     do;
/* Search for state */
     instate=0;
     do j=1 to nstat(i);
      if lstat(i,j) eq tgeos(mo,ip) then
       do;
        instate=1; /* found state, update frequency */
        fstat(i,j)=fstat(i,j)+1;
       end;
     end;
     if instate eq 0 then /* did not find state, add it */
       do;
       nstat(i)=nstat(i)+1;
       lstat(i,j)=tgeos(mo,ip); /* store the state */
       fstat(i,j)=fstat(i,j)+1; /* update frequency count */
       end;
     end;
  end;  /* end of month loop */
  end;  /* end of people loop */

/* Check to see if any HHADDIs have multiple states defined */
 if nstat(i) gt 1 then
/* When multiple states, use one with highest frequency */
  do;
  maxindx=1;
  maxfrq=fstat(i,1);
  do j=2 to nstat(i);
   if fstat(i,j) gt maxfrq then 
    do;
    maxindx=j;
    maxfrq=fstat(i,j);
    end;
  end;
  lstat(i,1)=lstat(i,maxindx);
  end;
 end;  /* end of code to build final HHADDI/STATE table */


/* FINAL PROCESSING AND OUTPUT SECTION *******************************/
/* Count total months person is single mom  */
/* Count total months person is in data with PPMIS=1 */
 do ip=1 to np;
 array nmsm{34};
 array nmt{34};
 nmsm(ip)=0; nmt(ip)=0;
 do mo=1 to 24;
 if sm(mo,ip) eq 1 and tpmis(mo,ip) eq 1 then nmsm(ip)=nmsm(ip)+1;
 if tpmis(mo,ip) eq 1 then nmt(ip)=nmt(ip)+1;
 end;
 end;


/* Output one record per month for each person in this PPID group */
 do ip=1 to np;  /* NP = total number of people in this PPID group */
 do mo=1 to 24;
/* Output data for months when PP-MIS = 1 or 2      */
/* (records with PPMIS=2 are only used when forming */
/* the Eligibility Criteria for Children and will   */
/* be deleted after that and not retained in data)  */
 if tpmis(mo,ip) in(1,2) then    
  do;
  ppentry=tentry(ip);
  pppnum=tnum(ip);
  ppmis=tpmis(mo,ip);
  sex=tsex(ip);
  ubrthmn=tbrmn(ip);
  ubrthyr=tbryr(ip);
  hhaddi=thhad(mo,ip);
  rrp=trrp(mo,ip);
  age=tage(mo,ip);
  ms=tms(mo,ip);
  famtyp=tfmtp(mo,ip);
  famrel=tfmrl(mo,ip);
  famnum=tfmnm(mo,ip);
  mnum=mo;
  date=xdate(mo);
  relat=relate(mo,ip);
  mompres=momp(mo,ip);
  dadpres=dadp(mo,ip);
  chpres=chp(mo,ip);
  othchldn=othc(mo,ip);
  numkids=nkid(mo,ip);
  ochrefgd=orfg(mo,ip);
  kidlt18=k18(mo,ip);
  kidlt12=k12(mo,ip);
  kidlt6=k6(mo,ip);
  kidlt2=k2(mo,ip);
  famstr=fstr(mo,ip);
  famid=fmid(mo,ip);
  npersns=npsn(mo,ip);
  oldest=old(mo,ip);
  youngest=yng(mo,ip);
  smom=sm(mo,ip);
  nummths=nmsm(ip);
  nmtot=nmt(ip);
/* Define STATE based on table created above */
  hnum=0;  /* HNUM = number of the HHADDI in the list */
  do i=1 to nlhh;  /* loop through all HHADDIs in list */
   if thhad(mo,ip) eq lhh(i) then hnum=i; /* set HNUM when find a match */
  end;
  if hnum ne 0 then state=lstat(hnum,1); /* use value from list */
  else put 'ERROR: HHADDI not in list when creating dataset '
   ppid ppentry pppnum mo ip thhad(mo,ip);


/* Create the person's SUNIQUE */
  syear='88';
  id1='000000000';
  id1=ppid;
  id2='00';
  id2=ppentry;
  id3='000';
  id3=pppnum;
  sunique=syear||id1||id2||id3;
/* If Parent information is available, create PARID (in form of SUNIQUE) */
  length parid $ 16;
  parguard=0; /* set Guardian flag to zero */
  parid='';
  if tpnpt(mo,ip) not in(0,.,999) and tentp(mo,ip) not in (0,.) then
  do;
   id2=tentp(mo,ip);
   id3=tpnpt(mo,ip);
   parid=syear||id1||id2||id3;
  end;
  else if tpngd(mo,ip) not in(0,.,999) and tengd(mo,ip) not in (0,.) then
  do;
   id2=tengd(mo,ip);
   id3=tpngd(mo,ip);
   parid=syear||id1||id2||id3;
   parguard=1; /* Set flag showing PARID comes from Guardian info */
  end;

 length rot ppentry pppnum ppmis sex
       ubrthmn ubrthyr hhaddi rrp age ms famtyp
       famrel famnum state mnum relat mompres dadpres chpres
       numkids kidlt18 kidlt12 kidlt6 kidlt2 oldest youngest
       ochrefgd othchldn
       famstr famid npersns smom nummths nmtot parguard 3;

  keep sunique rot ppid ppentry pppnum ppmis sex 
       ubrthmn ubrthyr hhaddi rrp age ms famtyp 
       famrel famnum state mnum date relat mompres dadpres chpres 
       numkids kidlt18 kidlt12 kidlt6 kidlt2 oldest youngest parid
       ochrefgd othchldn
       famstr famid npersns smom nummths nmtot parguard ;
  output;
  end;  /* end of PP-MIS = 1,2 block (tpmis(mo,ip) in(1,2) */
 end;   /* end of Month loop (mo=1 to 24) */
 end;    /* end of People loop (ip=1 to np) */

 end;   /* end of processing for last person in PPID group (last.ppid) */

/* Just for curiosity, show maximum number of families per PPID & */
/* maximum number of people per family (after reach end-of-file). */
 if end_file then
 do;
  file 'SIPP_families88.info';
  put 'MAXFAMILIES=' maxfam ' First Maxfam PPID is ' maxfamid 
      ' in month ' maxfammo ;
  put 'MAXPEOPLE PER FAMILY='  maxppl ' First Maxppl PPID is ' maxpplid
      ' in month ' maxpplmo ;
 end;
run;


proc freq; tables relat famstr smom nummths state 
           numkids ochrefgd othchldn chpres;
run;

/* Print some to check them */
options obs=100; 
proc print;
run;

endsas;
