options linesize=80 pagesize=66 nocenter;

libname x '';

/* Unlike the earlier SIPP data, the 1996 SIPP does not need */
/* any special processing of the STATE values because there  */
/* are no inconsistent PPID/HHADDI/STATE values in this data.*/

/* Definitions:                                              */
/*  Child         a person less than or equal to 18 years old*/
/*  KID           a "child of reference person" (RRP=4 for a */
/*                primary family or FAMREL=3 for a subfamily)*/
/*                and it does not include other children in  */
/*                the household (either related or unrelated)*/
/*  SingleMother  a female "reference person" with no spouse */
/*                present and at least one KID present       */
/*                during the month, with a MaritalStatus of  */
/*                3(widow), 4(divorced), 5(separated), or 6  */
/*                (never married).                           */

/* Program creates these new variables:                      */
/*                                                           */
/*  SMOM        0 = not a Single Mother in this month        */
/*              1 = a Single Mother in this month            */
/*                                                           */
/*  RELAT       1 = reference person (living with relatives) */
/*              2 = spouse of reference person               */
/*              3 = child of reference person                */
/*              4 = other relative of reference person       */
/*             14 = other relative of reference person, less */
/*                  than or equal to 18 years old, who has   */
/*                  reference person as their Guardian       */
/*              5 = "unrelated individual" (could be related */
/*                  to others in household) - corresponds to */
/*                  three categories of SIPP RRP variable:   */
/*                  (2) "reference person alone or with      */
/*                  non-relatives"; (6) "non-relative of     */
/*                  reference person but related to others   */
/*                  in household"; (7) "not related to anyone*/
/*                  else in household (secondary individual)"*/
/*                                                           */
/*  FAMSTR   (Family structure includes people with PPMIS=2) */
/*              0 = individuals living alone                 */
/*              1 = single mom with kids                     */
/*              2 = two parents with kids                    */
/*              3 = single dad with kids                     */
/*              4 = female head, no kids                     */
/*              5 = dual head, no kids                       */
/*              6 = male head, no kids                       */
/*              7 = no adult head, but with kids             */
/*                                                           */
/*  MOMPRES     0 = no female ref person or spouse this month*/
/*              1 = female ref person or spouse present      */
/*              2 = female ref person but with PPMIS=2       */
/*  DADPRES     0 = no male ref person or spouse this month  */
/*              1 = male ref person or spouse present        */
/*              2 = male ref person but with PPMIS=2         */
/*                                                           */
/*  NUMKIDS     = total number of KIDS in family this month  */
/*              (i.e., RELAT=3 children)                     */
/*                                                           */
/*  OCHREFGD    = total number of RELAT=14 children present  */
/*              this month (i.e., other relative of RefPerson*/
/*              who has RefPerson as their Guardian)         */
/*                                                           */
/* Note: In all the following variables, "kid" includes both */
/* RELAT=3 children (child of reference person) and RELAT=14 */
/* children (other relative of reference person, who has the */
/* RefPerson as their Guardian                               */ 
/*  KIDLT18     = number of kids less than 18                */
/*  KIDLT12     = number of kids less than 12                */
/*  KIDLT6      = number of kids less than  6                */
/*  KIDLT2      = number of kids less than  2                */
/*  OLDEST      = age of oldest kid in family this month     */
/*  YOUNGEST    = age of youngest kid in family this month   */
/*                                                           */
/*                                                           */
/*  OTHCHLDN    = number of other children present this month*/
/*              in addition to those with RELAT=3,14         */
/*              which includes people less than or equal to  */
/*              18 years old with RELAT=4 (other relative of */
/*              reference person) or RELAT=5 (unrelated)     */
/*                                                           */
/*  CHPRES      = total number of related children present   */
/*              this month, where "related child" includes   */
/*              KID as defined above and also includes people*/
/*              less than or equal to 18 years with RELAT=4  */
/*              or =14 (other relative of reference person)  */
/*                                                           */
/*  FAMID       = family number for PPID/HHADDI in a month   */
/*  NPERSNS     = number of people in family this month      */
/*                                                           */


/* Define macro to read raw data for each wave of the 1996 SIPP. */
%macro readraw;
%do nwav = 1 %to 12;
data tmp&nwav;
 filename yy "c:\SIPP_data\sipp96l&nwav..dat";
 missing A;
 infile yy lrecl = 20000 missover ;

 input 
/* note that PPID size was 9 in earlier years) */
@6	PPID    12. /* SSUID     - named SUID in pre-93 core */
@24	ROT      1. /* SROTATON */
@25	INTMON   1. /* SREFMON  */
@26     RHCALMN  2.
@28     RHCALYR  4.
@32	HHADDI   3. /* SHHADID   (size was 2 in earlier years)*/
@42	GEOSTE	 2. /* TFIPSST   */
@253	RFID     3. /* Family ID number */
@256	RFID2    3. /* Family ID excluding related subfamily members */
@385	FAMNUM   3. /* RSID - Related or Unrelated subfamily ID number */
@511	PPENTRY  3. /* EENTAID   - named ENTRY in pre-93 core */
@514	PPPNUM   4. /* EPPPNUM   */
/*@	PPMIS    1.*/ /* Not needed: use ALL person-month records */
@521	UBRTHMN  2. /* EBMNTH    */
@524	UBRTHYR  4. /* TBYEAR    */
@529	SEX      1. /* ESEX      */
@571	FAMREL	 1. /* ESFR      */
@572	FAMTYP	 1. /* ESFT      */
@573	AGE	 2. /* TAGE      */
@576	RRP	 2. /* ERRP       Values different than pre-93!! */
@579	MS	 1. /* EMS       */
@581	PNSP	 4. /* EPNSPOUS  */
/*@1746	ENTSP	 2.  Not Needed in 1996 SIPP */
/* @1818	PNPT	 3.*/  /* use EPNMOM and EPNDAD  */
/* @1926	ENTPT	 2.  Not Needed in 1996 SIPP */
@586	pnmom    4. /* EPNMOM - PersonNumber of Mom */
@591	pndad    4. /* EPNDAD - PersonNumber of Dad */
@596	pngd     4. /* EPNGUARD  */
/* @2103   entgd    2.  Not Needed in 1996 SIPP */
;

/* Define month number (1996 SIPP has 12 4-months waves) */
/* as InterviewMonth plus WaveNumber-1 times 4 */
 mnum=intmon+ ( &nwav -1)*4; 
 
run;

/* Append each of the 12 temporary datasets to big file. */
proc datasets nolist;
  append out=sip96l  new=tmp&nwav;  delete tmp&nwav;
run;

/* End of loop */
%end;

/* End of macro */
%mend readraw;

/* Invoke the macro to loop through all 12 raw data files. */
%readraw;

/* Sort in order by PPID, reference month, and PPPNUM  */
/* (PPENTRY not needed in 1996 SIPP)                   */
proc sort data=sip96l; by ppid mnum pppnum;
run;

/* Look at all records for a PPID for each month  */
/* and build a matrix of all it's people.         */
data x.fam96;
 set sip96l end=end_file; by ppid mnum;

/* Define a series of temporary arrays to hold the data. */
/* There are 4 months in each data file for the 1996 SIPP */
/* data, and each month is handled separately.            */ 
/* Allow for a maximum of 29 people in each PPID group.   */
 array tentry{29} _temporary_ ;
 array tnum{29} _temporary_  ;
 array tsex{29} _temporary_ ;
 array tbrmn{29} _temporary_ ;
 array tbryr{29} _temporary_ ;
 array thhad{29} _temporary_ ;
 array trrp{29} _temporary_ ;
 array tage{29} _temporary_ ;
 array tms{29} _temporary_ ;
 array tfmtp{29} _temporary_ ;
 array tfmrl{29} _temporary_ ;
 array tpnsp{29} _temporary_ ;
 array tpnmom{29} _temporary_ ;
 array tpndad{29} _temporary_ ;
 array tpngd{29} _temporary_ ;
 array tfid{29} _temporary_ ;
 array tfid2{29} _temporary_ ;
 array tsid{29} _temporary_ ;
 array tgeos{29} _temporary_ ;

/* NP is the person number of someone in this PPID group */
 retain np;
 if first.ppid or first.mnum then np=1; else np=np+1;

/* Put person's identifying info into temporary array */
 tentry(np)=ppentry;
 tnum(np)=pppnum;
 tsex(np)=sex;
 tbrmn(np)=ubrthmn;
 tbryr(np)=ubrthyr;

/* Create date in YYMM format (to match old files). */
 if rhcalyr gt 1999 then
   date=((rhcalyr-2000)*100)+rhcalmn;
 else date=((rhcalyr-1900)*100)+rhcalmn;
 drop rhcalyr rhcalmn;

/* Put person data into temporary array*/
  thhad(np)=hhaddi;
  trrp(np)=rrp;
  tage(np)=age;
  tms(np)=ms;
  tfmtp(np)=famtyp;
  tfmrl(np)=famrel;
  tpnsp(np)=pnsp;
  tpnmom(np)=pnmom;
  tpndad(np)=pndad;
  tpngd(np)=pngd;
  tfid(np)=rfid;   /* TFID/TFID2/TSID are "Family Variables" from raw data */
  tfid2(np)=rfid2;
  tsid(np)=famnum;
  tgeos(np)=geoste;

/* Do special things when reach last person in the PPID group  */

/* NOTE: This version that uses the RFID/RFID2/FAMNUM (i.e., RSID) */
/* from raw data rather than computing values for these variables  */
/* (which was necessary for all previous SIPP years).              */

 if last.ppid or last.mnum then
 do;

/* FAMILY DEFINITION SECTION *****************************************/
/* For each month, identify who makes up a family */
/* Create array FAMLY - each column contains the person number  */
/* of all the members of one family in this PPID group.         */
/* Variable NUMFAMLY = the number of families in the PPID group */
/* Array NPERSN contains the number of people in each family    */
 array famly{29,29};  /* could be 29 1-person families or 1 29-person family */
 array npersn(29);     /* count of number of persons in each family */
 do i=1 to 29;
 npersn(i)=0;
 do j=1 to 29;
 famly(i,j)=0;
 end;
 end;
/* Keep track of maximum number of families per PPID */
/* and maximum number of people per family.          */
 retain maxfam maxfamid maxfammo maxppl maxpplid maxpplmo;
 if _n_ eq 1 then do;maxfam=1;maxfamid=ppid;maxfammo=1;
                     maxppl=1;maxpplid=ppid;maxpplmo=1;end;
/* Put first person into "family #1" */
 numfamly=1;
 npersn(1)=0;
 do ip=1 to np while (famly(1,1) eq 0);
  if (tsid(ip) not in (-1,0,.) or tfid2(ip) not in(-1,0,.) ) then
    do;
    npersn(1)=1;
    famly(numfamly,npersn(1))=ip;
    end;
 end;
/* Are there any people at all in this month? */
 if npersn(1) eq 0 then goto nobody;
/* If someone, check to be sure that 1st person has an address */
 prsn1= famly(numfamly,npersn(1));
 if  thhad(prsn1) in (0,.) then
  do;
  put 'ERROR: ' ppid tentry(prsn1) tnum(prsn1) ' Person #1 has no address!';
  put 'month ' mnum 'sid,fid2= ' tsid(prsn1) tfid2(prsn1);
  put ' hhaddi= ' thhad(prsn1);
  end;

/* Check all the other people in that month */
 do ip=(prsn1+1) to np;
    match1=0;
    match2=0;
/* Is this person (not in related subfamily) in a family we've already seen? */
  if tfid2(ip) not in (-1,0,.) then
    do i=1 to numfamly;
      if tfid2(ip) eq tfid2(famly(i,1)) and
         thhad(ip) eq thhad(famly(i,1)) then match1=i;
    end;
/* Is this person in a related subfamily that we've already seen? */
  if tsid(ip) not in(-1,0,.) and tfid2(ip) in (-1,0,.) then
    do i=1 to numfamly;
      if tsid(ip)  eq tsid(famly(i,1))  and
         thhad(ip) eq thhad(famly(i,1)) then match2=i;
    end;
/* If there was a match, add this person to existing family */
  if match1 ne 0 then   /* Match was NOT in a related subfamily */
    do;
    npersn(match1)=npersn(match1)+1;    /* Increment # people in family */
    /* Keep track of maximum number of people in any family */
    if  npersn(match1) gt maxppl then do;
        maxppl=npersn(match1); maxpplid=ppid; maxpplmo=mnum; end;
    famly(match1,npersn(match1))=ip;    /* Put person # in family array */
    end;
  if match2 ne 0 then   /* Match WAS in a related subfamily */
    do;
    npersn(match2)=npersn(match2)+1;    /* Increment # people in family */
    /* Keep track of maximum number of people in any family */
    if  npersn(match2) gt maxppl then do;
        maxppl=npersn(match2); maxpplid=ppid; maxpplmo=mnum; end;
    famly(match2,npersn(match2))=ip;    /* Put person # in family array */
    end;
/* If there was not a match, start another family */
  if match1 eq 0 and match2 eq 0 then
    do;
    numfamly=numfamly+1;
    npersn(numfamly)=1;
    famly(numfamly,1)=ip;
/* Keep track of maximum number of families in any PPID group */
    if numfamly gt maxfam then do;
       maxfam=numfamly; maxfamid=ppid; maxfammo=mnum; end;
    end;
 end;  /* end of People loop (ip=(prsn1+1) to np) */

/* Check that all the people in each "family" are living */
/* together at the same address that month.              */
 do i=1 to numfamly;
 sameadd=1;
 if npersn(i) gt 1 then
  do j=2 to npersn(i);
   if thhad(famly(i,j)) ne thhad(famly(i,j-1)) then sameadd=0;
  end;
 if sameadd ne 1 then put 'ERROR: Family has different addresses.'
   ' Month=' mnum '  PPID=' ppid '  FID2/SID=' famly(i,1);
 end;  /* end of Family loop (i=1 to numfamly) */

/* Loop through each person in "family" and identify their role  */
/* (1=head,2=spouse,3=kid,4=other rel,5=unrelated individual)  */
 do i=1 to numfamly;
 do j=1 to npersn(i);
 array relate{29} _temporary_ ;
/* Subfamily */
 if tfmtp(famly(i,j)) eq 3           /* Related subfamily */
   or tfmtp(famly(i,j)) eq 2         /* Unrelated subfamily */
   then relate(famly(i,j))=tfmrl(famly(i,j));
/* Not a Subfamily */
 if tfmtp(famly(i,j)) in (0,1,4) then
   do;
   if trrp(famly(i,j)) eq 1 then
     relate(famly(i,j))=1;  /* reference person */
   if trrp(famly(i,j)) eq 3 then
     relate(famly(i,j))=2;  /* spouse */
   if trrp(famly(i,j)) eq 4 then
     relate(famly(i,j))=3;  /* kid */
   if trrp(famly(i,j)) in (5,6,7,8) then
     relate(famly(i,j))=4;  /* other relative */
   if trrp(famly(i,j)) in (2,9,10,11,12,13) then
     relate(famly(i,j))=5;  /* unrelated individual */
   /* Note RELAT=5 could be related to others in household */
   end;
 end;  /* end of People in Family loop (j=1 to npersn(i)) */
 end;  /* end of Families loop (i=1 to numfamly) */

/* Loop through each person in "family"; set flags, count kids */
 do i=1 to numfamly;
/* Check Person_number of Parent (PNPT) to see if (1) each KID's */
/* matches the ref person (or spouse) in this month and (2) any  */
/* OtherRelative or NonRelative matches the ref person or spouse.*/
/* (Need separate loop: parent might be after child in order)    */
 file 'SIPP_families96.conflicting_parentinfo';
/* First find family reference person and/or spouse */
 rpnpt=.; rentpt=.; spnpt=.; sentpt=.; /* initialize for family */
 do j=1 to npersn(i);
 ip=famly(i,j); /* person number of person in this family */
 if relate(ip) eq 1 then  /* if it's the reference person, then */
   do;
   rpnpt=tnum(ip);         /* store their PPPNUM  */
   rpsex=tsex(ip);         /* store sex of RefPerson */
   end;
 if relate(ip) eq 2 then  /* if it's the spouse of ref person, then */
   do;
   spnpt=tnum(ip);         /* store their PPPNUM  */
   end;
 end;
/* Next check kids */
 do j=1 to npersn(i);
 ip=famly(i,j); /* person number of person in this family */
 if relate(ip) eq 3  /* child of ref person (i.e, KID) */
   and tage(ip) le 18 /* child not older than 18 */
   and (tpnmom(ip) not in (0,9999) or
        tpndad(ip) not in (0,9999)) /* there is person# of parent */
  then  /* check if that person# is RefPerson/Spouse this month */
  do;
   pmatch=0;    /* flag set to "no match" with parent Pnum */
   if ((tpnmom(ip) eq rpnpt or tpndad(ip) eq rpnpt) /* match ref */
     or (tpnmom(ip) eq spnpt or tpndad(ip) eq spnpt))  /* match spouse */
      then pmatch=1; /* PNMOM or PMDAD is a valid match */
   if pmatch eq 0 then
     do;
     put "KID's PNMOM/PNDAD does not match RefPerson or Spouse this month";
     put 'PPID=' ppid ' PPENTRY=' tentry(ip) ' PPPNUM=' tnum(ip)
      'Month=' mnum ' PNMOM=' tpnmom(ip) ' PNDAD=' tpndad(ip);
    /* ?? tpnpt(ip)=0; ?? */  /* reset PNPT to 0 */
     end;
  end;  /* end of KID in Family block (relate(ip) eq 3) */
/* Also check if person is "other relative or nonrelative" but */
/* has parent information saying the ref person is their parent*/
 if relate(ip) in (4,5)  /* other rel or non rel of ref person */
   and tage(ip) le 18 /* child not older than 18 */
   and (tpnmom(ip) not in (0,9999) or
        tpndad(ip) not in (0,9999)) /* there is person# of parent */
  then  /* check if that person# is RefPerson/Spouse this month */
  do;
   pmatch=0;    /* flag set to "no match" with parent Pnum */
   /* Loop through all people in family this month */
   do k=1 to npersn(i);
     if (tpnmom(ip) eq tnum(famly(i,k)) or
         tpndad(ip) eq tnum(famly(i,k))) and /* person# matches */
      relate(famly(i,k)) in (1,2)         /* ref person or spouse */
      then pmatch=1;
   end;
   if pmatch eq 1 then
     do;
     put "OtherRelative or Unrelated PNMOM/PNDAD matches ref person or spouse";
     put 'PPID=' ppid  ' PPPNUM=' tnum(ip)
      'Month=' mnum ' PNMOM=' tpnmom(ip) ' PNDAD=' tpndad(ip);
     put 'IP=' ip ' RELATE(IP)=' relate(ip) ' PMATCH=' pmatch; 
     end;
  end;  /* end of Other/Unrelat block (relate(ip) in (4,5) ) */
/* Finally, check if person is "other relative or nonrelative",  */
/* has NO parent information, and has Guardian information saying */
/* the RefPerson or Spouse of RefPerson is their Guardian         */
 if relate(ip) in (4,5)  /* other rel or non rel of ref person */
   and tage(ip) le 18 /* child not older than 18 */
   and tpnmom(ip) in (0,9999) and
       tpndad(ip) in (0,9999) /* there is NO person# of parent */
   /* and the RefPerson/Spouse is designated as the child's Guardian */
   and ((tpngd(ip) eq rpnpt)
     or (tpngd(ip) eq spnpt))
  then  /* mark this child as having that RefPerson/Spouse as Guardian */
   relate(ip)=relate(ip)+10; /* recode RELAT to 14 */
 end;  /* end of People in Family loop (j=1 to npersn(i)) */

/* If there is no parent defined for a "Child of RefPerson" via      */
/* PNMOM/PNDAD, then define reference person (RELAT=1) as the parent */
 do j=1 to npersn(i);
 ip=famly(i,j); /* person number of person in  this family */
 if relate(ip) eq 3  /* child of ref person (i.e, KID) */
   and tage(ip) le 18 /* child not older than 18 */
   and (tpnmom(ip) in (0,9999)  /* there is NO person# of mom */
     and tpndad(ip) in (0,9999)) /* there is NO person# of dad */
  then
  do;
  if rpsex eq 1 then
   tpndad(ip)=rpnpt;
   else tpnmom(ip)=rpnpt;
  end;
 end;

/* Initialize variables */
 mompres=0;
 dadpres=0;
 chpres=0;
 numkids=0;
 othchldn=0;
 ochrefgd=0;
 kidlt18=0;
 kidlt12=0;
 kidlt6=0;
 kidlt2=0;
 oldest=.;
 youngest=.;
 do j=1 to npersn(i);
  if relate(famly(i,j)) in (1,2) and tsex(famly(i,j)) eq 1
     then dadpres=1;
  if relate(famly(i,j)) in (1,2) and tsex(famly(i,j)) eq 2
     then mompres=1;
  if relate(famly(i,j)) eq 3 and tage(famly(i,j)) le 18 then
    numkids=numkids+1; /* Number of RELAT=3 children in family */
  if relate(famly(i,j)) eq 14 and tage(famly(i,j)) le 18 then
    ochrefgd=ochrefgd+1; /* Number of RELAT=14 children in family */
/* treat RELAT=14 children just like REAL KIDS when summing them */
  if relate(famly(i,j)) in (3,14)
   and tage(famly(i,j)) le 18 then     /* KIDs block */
    do;
    if tage(famly(i,j)) lt 18 then kidlt18=kidlt18+1;
    if tage(famly(i,j)) lt 12 then kidlt12=kidlt12+1;
    if tage(famly(i,j)) lt 6 then kidlt6=kidlt6+1;
    if tage(famly(i,j)) lt 2 then kidlt2=kidlt2+1;
    if youngest eq . or tage(famly(i,j)) lt youngest then
       youngest=tage(famly(i,j));
    if oldest eq . or tage(famly(i,j)) gt oldest then
       oldest=tage(famly(i,j));
    end;  /* end of KIDS block */
  if relate(famly(i,j)) in (4,5) and tage(famly(i,j)) le 18 then
    othchldn=othchldn+1; /* Additional children (RELAT=4,5) in family */
  if relate(famly(i,j)) in (3,4,14) and tage(famly(i,j)) le 18 then
    chpres=chpres+1;  /* all children in family related to RefPerson */
 end;  /* end of People in Family loop (j=1 to npersn(i)) */

/* Define the family structure */
 famstr=0;                      /* individuals living alone */
 if mompres ne 0 and dadpres eq 0 and (numkids+ochrefgd) gt 0
    then famstr=1;              /* single mom with kids */
 if mompres ne 0 and dadpres ne 0 and (numkids+ochrefgd) gt 0
    then famstr=2;              /* two parents with kids */
 if mompres eq 0 and dadpres ne 0 and (numkids+ochrefgd) gt 0
    then famstr=3;              /* single dad with kids */
 if mompres ne 0 and dadpres eq 0 and (numkids+ochrefgd) eq 0
    then famstr=4;              /* female head, no kids */
 if mompres ne 0 and dadpres ne 0 and (numkids+ochrefgd) eq 0
    then famstr=5;              /* dual head, no kids */
 if mompres eq 0 and dadpres ne 0 and (numkids+ochrefgd) eq 0
    then famstr=6;              /* male head, no kids */
 if mompres eq 0 and dadpres eq 0 and (numkids+ochrefgd) gt 0
    then famstr=7;              /* no adult head, but with kids */
/* Store the family information in each person's data */
 array momp{29} _temporary_ ;
 array dadp{29} _temporary_ ;
 array chp{29} _temporary_ ;
 array othc{29} _temporary_ ;
 array nkid{29} _temporary_ ;
 array orfg{29} _temporary_ ;
 array k18{29} _temporary_ ;
 array k12{29} _temporary_ ;
 array k6{29} _temporary_ ;
 array k2{29} _temporary_ ;
 array old{29} _temporary_ ;
 array yng{29} _temporary_ ;
 array fstr{29} _temporary_ ;
 array sm{29} _temporary_ ;
 array fmid{29} _temporary_ ;
 array npsn{29} _temporary_ ;
 do j=1 to npersn(i);
  momp(famly(i,j))=mompres;
  dadp(famly(i,j))=dadpres;
  chp(famly(i,j))=chpres;
  othc(famly(i,j))=othchldn;
  nkid(famly(i,j))=numkids;
  orfg(famly(i,j))=ochrefgd;
  k18(famly(i,j))=kidlt18;
  k12(famly(i,j))=kidlt12;
  k6(famly(i,j))=kidlt6;
  k2(famly(i,j))=kidlt2;
  old(famly(i,j))=oldest;
  yng(famly(i,j))=youngest;
  fstr(famly(i,j))=famstr;
  fmid(famly(i,j))=i;
  npsn(famly(i,j))=npersn(i);
  ip=famly(i,j); /* person number of person in this family */
  sm(ip)=0;   /* Not a SingleMother this month */
  if fstr(ip) eq 1           /* Family Structure "Single mom with kids" */
    and tsex(ip) eq 2           /* female */
    and relate(ip) in (1,2)  /* either reference person or spouse of ref */
    and tms(ip) in (3,4,5,6) /* only MS widow,divorce,separated,nevermar */
    then sm(ip)=1;           /* Person IS a SingleMother this month */
 end;  /* end of People in Family loop (j=1 to npersn(i)) */
 end;  /* end of Families loop (i=1 to numfamly) */
nobody: ; /* jump here if cannot put even 1 person into family#1 this month */

/* FINAL PROCESSING AND OUTPUT SECTION *******************************/

/* Output one record per month for each person in this PPID group */
 do ip=1 to np;  /* NP = total number of people in this PPID group */
  ppentry=tentry(ip);
  pppnum=tnum(ip);
  sex=tsex(ip);
  ubrthmn=tbrmn(ip);
  ubrthyr=tbryr(ip);
  hhaddi=thhad(ip);
  rrp=trrp(ip);
  age=tage(ip);
  ms=tms(ip);
  famtyp=tfmtp(ip);
  famrel=tfmrl(ip);
  famnum=tsid(ip);
  pnmom=tpnmom(ip);
  pndad=tpndad(ip);
  relat=relate(ip);
  mompres=momp(ip);
  dadpres=dadp(ip);
  chpres=chp(ip);
  othchldn=othc(ip);
  numkids=nkid(ip);
  ochrefgd=orfg(ip);
  kidlt18=k18(ip);
  kidlt12=k12(ip);
  kidlt6=k6(ip);
  kidlt2=k2(ip);
  famstr=fstr(ip);
  famid=fmid(ip);
  npersns=npsn(ip);
  oldest=old(ip);
  youngest=yng(ip);
  smom=sm(ip);
  state=tgeos(ip);

/* Create the person's SUNIQUE */
  syear='96';
  id1='000000000000';
  id1=ppid;
  id3='0000';
  id3=pppnum;
  sunique=syear||id1||id3;
/* If Parent information is available, create PARID (in form of SUNIQUE) */
  length parid $ 18;
  parguard=0; /* set Guardian flag to zero */
  parid='';
  if tpnmom(ip) not in(-1,0,.,9999) then 
  do;
   id3=tpnmom(ip); 
   parid=syear||id1||id3;
  end;
  else if tpndad(ip) not in(-1,0,.,9999) then
  do;
   id3=tpndad(ip); 
   parid=syear||id1||id3;
  end;
  else if tpngd(ip) not in(-1,0,.,9999)  then
  do;
   id3=tpngd(ip);
   parid=syear||id1||id3;
   parguard=1; /* Set flag showing PARID comes from Guardian info */
  end;

 length rot ppentry pppnum sex
       ubrthmn ubrthyr hhaddi rrp age ms famtyp
       famrel famnum state mnum relat mompres dadpres chpres
       numkids kidlt18 kidlt12 kidlt6 kidlt2 oldest youngest
       ochrefgd othchldn
       famstr famid npersns smom parguard intmon 3;
  length pnmom pndad 4;

  keep sunique rot ppid ppentry pppnum sex
       ubrthmn ubrthyr hhaddi rrp age ms famtyp
       famrel famnum state mnum date relat mompres dadpres chpres
       numkids kidlt18 kidlt12 kidlt6 kidlt2 oldest youngest parid
       ochrefgd othchldn pnmom pndad
       famstr famid npersns smom parguard intmon;
  output;

 end;    /* end of People loop (ip=1 to np) */
 end;   /* end of processing for last person in PPID group (last.ppid) */

/* Just for curiosity, show maximum number of families per PPID & */
/* maximum number of people per family (after reach end-of-file). */
 if end_file then
 do;
  file 'SIPP_families96.info';
  put 'MAXFAMILIES=' maxfam ' First Maxfam PPID is ' maxfamid
      ' in month ' maxfammo ;
  put 'MAXPEOPLE PER FAMILY='  maxppl ' First Maxppl PPID is ' maxpplid
      ' in month ' maxpplmo ;
 end;
run;

/* Put into correct order to match earlier SIPP years: */
/* Sort in order by PPID, PPPNUM, and reference month  */
/* (PPENTRY not needed in 1996 SIPP)                   */
proc sort data=x.fam96; by ppid pppnum mnum;
run;

proc freq; tables mnum* intmon /missing norow nocol nopercent;
run;

proc freq; tables relat famstr smom state
           numkids ochrefgd othchldn chpres;
run;

/* Print some to check them */
options obs=100;
proc print;
run;


endsas;

