options linesize=80 nocenter pagesize=66;
options formdlim=' ' nodate nonumber;

/* This program is run:                                         */
/* - after SIPP_families96.sas has created a family structure   */
/*   from the main SIPP data (dataset fam96.sd2) and            */
/* - after kdisab96.sas has created a Kid Disability dataset    */
/*   (kdisab96.sd2).                                            */
/* - after adisab96.sas has created an Adult Disability dataset */
/*   (adisab96.sd2).                                            */
/* - after immig96.sas has created an Immigration dataset       */
/*   (immig96.sd2).                                             */

libname family '';

/* Define macro to read raw data for each wave of the 1996 SIPP.    */
%macro readraw;
%do nwav = 1 %to 12;

/* Get Income & Program-Participation variables from 1996 SIPP data */
/* (these were NOT read when the family structure was created).     */
/* 
options obs=100;
 */

data tmp&nwav; 
filename yy "c:\SIPP_data\sipp96l&nwav..dat";
 missing A;
 infile yy lrecl = 20000 missover pad end=eof ;

 input
@6      PPID    12. /* SSUID    */
@25     INTMON   1. /* SREFMON  */
@514    PPPNUM   4. /* EPPPNUM  */

/* 1996 SIPP files do not have PNLWGT and FNLWTyy values in */
/* the data, only monthly weights.  The Panel Weight and    */
/* Weight-for-Year values were made available on the census */
/* Bureau web site as a seperate file.  These values are    */
/* merged in during the following data step.                */
@561	WPFINWGT 10.4 /* WW: Person weight - monthly value  */

@573	AGE      2. /* TEMPORARY - FOR DEBUGGING ONLY */
@531	RACE     1. /* ERACE    PE: Race of this person */
/*      Note 1996 SIPP ethnicity values are different than pre-96!! */
@533	ETHNCTY  2. /* EORIGIN  PE: Origin of this person */

@621	PPEARN   7. /* TPEARN   PE: Total person's earned income for month */
@650	PPINC    8. /* TPTOTINC PE: Total person's income for month */  
/*      Note 1996 SIPP education values are different that pre-96 values! */
@754	EENRLM   2. /* EENRLM   ED: Enrollment status in this month */
@759	EENLEVEL 2. /* EENLEVEL ED: Level or grade enrolled */
@786    EEDUCATE 2. /* EEDUCATE ED: Highest Degree received / grade completed */

@811	WDISAB   2. /* EDISABL  LF: Physical/mental work-limiting condition */
@814	WDISABP  2. /* EDISPREV LF: Had work-preventing condition */

/* 1996 SIPP does not have a WAVFLG variable. */
/* @nnnn (WAVFLG1-WAVFLG36) (1.)              */

@855	ESR      2. /* RMESR    LF: Employment status recode for month */
@879	JOBID1   2. /* EENO1    JB: Across-wave employer index/number */
@955	JOBID2   2. /* EENO2    JB: Across-wave employer index/number */
@1026	BUSID1   2. /* EBNO1    BS: Across-wave business index/number */
@1110	BUSID2   2. /* EBNO2    BS: Across-wave business index/number */

/* Income Source Code 3: SSI (Supplemental Security Income) */
@1341	SSIREC   2. /* ER03A  GI: Receipt of Federal SSI - Adult (ISS Code 3) */
/*      Note: ignore @1218 ESSISELF "GI: Receipt of SSI for self    */
/*      (ISS Code 3)" because it has one value per wave rather      */
/*      than one value per month.                                   */
@1473	SSIAMT   5. /* T03AMTA GI: Amount of Federal SSI - Adult (ISS Code 3) */
@1478	SSIIMP   1. /* A03AMTA  GI: Allocation flag for T03AMTA */
@1344   ssirecch 2. /* ER03K  GI: Receipt of Federal SSI - Child (ISS Code 3) */
/*      Note: ignore @1215 ESSICHLD "GI: Receipt of SSI for children*/
/*      (ISS Code 3)" because it has one value per wave rather      */
/*      than one value per month.                                   */
@1479   ssiamtch 5. /* T03AMTK GI: Amount of Federal SSI - Child (ISS Code 3) */
@1484	ssiimpch 1. /* A03AMTK  GI: Allocation flag for T03AMTK */
@1347	ssirecst 2. /* ER04     GI: Receipt of State SSI (ISS Code 4) */
/*      Note: ignore @1221 ESTSSI "GI: Receipt of State administered */ 
/*      SSI (ISS Code 4)" because it has one value per wave rather   */
/*      than one value per month.                                    */
@1349	ssircimp 1. /* AR04     GI: Allocation flag for ER04 */
@1485	ssiamtst 5. /* T04AMT   GI: Amount of State SSI (ISS Code 4) */
@1490	siiimpst 1. /* A04AMT   GI: Allocation flag for T04AMT */
/* There are additional variables for SSI coverage:  */
@680	RCUTYP03 1. /* PE: Federal SSI coverage flag */
@685	RCUTYP04 1. /* PE: State SSI coverage flag */

/* Income Source Code 20: AFDC (Aid to Families with Dependent Children) */
@699    AFDC     1. /* RCUTYP20 PE: AFDC program coverage flag */
@700	RCUOWN20 4. /* Person number of the owner of AFDC coverage */
@1377   AFDCREC  2. /* ER20     GI: Receipt of AFDC (ISS Code 20) */
/*      Note: ignore @1290 EPATYP1 "GI: Receipt of AFDC (ISS Code 20)" */
/*      because it has one value per wave rather than one per month.   */
@1545	PERSAFDC 5. /* T20AMT   GI: Amount of AFDC (ISS Code 20) */
@1550	AFDCIMP  1. /* A20AMT   GI: Allocation flag for T20AMT */

/* Income Source Code 21: General Assistance */
@1380	GAREC    2. /* ER21     GI: Receipt of General Assistance/Relief */
/*      Note: ignore @1293 PATYP2 "GI: Receipt of general assistance */ 
/*      (ISS Code 21)" because it has one value per wave rather than */
/*      one value per month.                                         */
@1551	GENASST  5. /* T21AMT   GI: Amount of General Assistance/Relief */
@1556	GAIMP    1. /* A21AMT   GI: Allocation flag for T21AMT */
@704	GASST    1. /* RCUTYP21 PE: General Assistance coverage flag */

/* Income Source Code 24: Other Welfare */
@1386	OTHWREC  2. /* ER24     GI: Receipt of Other Welfare (ISS Code 24)  */
/*      Note: ignore @1299 EPATYP4 "GI: Receipt of other public assistance  */
/*      (ISS Code 24)", @1305 EPATYP6 "GI: Receipt of Short-Term Cash (ISS  */
/*      Code 24)", @1308 EPATYP7 "GI: Receipt of Other Government Assistance*/
/*      (ISS Code 24) - all are per-wave ISS code 24 values.                */
@1563	OTHWELF  5. /* T24AMT   GI: Amount of other welfare (ISS Code 24) */
@1568	OTHWIMP  1. /* A24AMT   GI: Allocation flag for T24AMT */
/* There is an additional variable for Other Welfare coverage:  */
@718	RCUTYP24 1. /* PE: Other welfare coverage flag */

/* Coverage flags */
@2099	MCARE    2. /* ECRMTH   HI: Medicare coverage in this month */
@2104	MCAID    2. /* ECDMTH   HI: Medicaid coverage in this month */
/*      Note: There is a second variable in the 1996 SIPP that has  */
/*      exactly the same values as ECDMTH for every case in every   */
/*      wave - ignore:  '@737  RCUTYP57 PE: Medicaid coverage flag' */
@732	FOODST   1. /* RCUTYP27 PE: Food Stamp coverage flag */
/*      Note: ignore @1278 EFSYN "GI: Receipt of food stamps (ISS  */
/*      Code 27)" because it has one value per wave rather than    */
/*      one value per month.                                       */

/*      Was ... covered by CHAMPUS, CHAMPVA/VA or military health care */
/*      coverage in this month?                                        */
@2123	CHAMP    2. /* RCHAMPM  HI: Military related health care coverage */
@2119	EHIOWNER 2. /* HI: Covered by own plan or someone else's */
/*      1  .Covered in own name                                  */
/*      2  .Covered by someone else's plan                       */
/*      3  .Covered both in own name and by someone else's plan  */
/*      4  .Not covered                                          */
@2122	ENONHH   1. /* HI: Covered by plan owned by person outside household */
@2116	EHIMTH   2. /* HI: Private health insurance coverage in this month */
/*      Note: There is a second variable in the 1996 SIPP that has         */
/*      exactly the same values as EHIMTH for every case in every          */
/*      wave - ignore:  '@742  RCUTYP58 PE: Health Insurance coverage flag'*/
@2134	EHEMPLY    2. /* HI: Source of health insurance */
/*      -1 .Not in universe                             */
/*       1 .Current employer or work                    */
/*       2 .Former employer                             */
/*       3 .Union                                       */
/*       4 .CHAMPUS                                     */
/*       5 .CHAMPVA                                     */
/*       6 .Military/VA health care                     */
/*       7 .Privately purchased                         */
/*       8 .Other                                       */
@2137	EHICOST    2. /*HI: Employer/union paid all or part health ins. costs*/
/*       1 .All    */
/*       2 .Part   */
/*       3 .None   */
;

/* Define month number (1996 SIPP has 12 4-months waves) */
/* as InterviewMonth plus WaveNumber-1 times 4 */
 mnum=intmon+ ( &nwav -1)*4;
run;

/* For waves 9 through 12 of the 1996 SIPP, find all people */
/* who are covered by AFDC but owner of the AFDC coverage   */
/* is another person (e.g., parent gets AFDC for child).    */
data tmpafdc;
 set tmp&nwav;
/* Only Waves 9-12 should get AFDC values changed, but */
/* check all the waves to be sure this is true.        */
 if afdc eq 1 and rcuown20 ne pppnum;
 keep ppid rcuown20 mnum;
run;
/* Rename the person number of the AFDC owner */
data tmpafdc;
 set tmpafdc;
 rename rcuown20=pppnum;
run;
proc sort data=tmpafdc nodupkey; by ppid pppnum mnum;
proc sort data=tmp&nwav; by ppid pppnum mnum;
run;

/* Add the information on AFDC owner to the other data */
/* and finish recoding and adding computed variables.  */
data tmp&nwav;
 merge tmp&nwav(in=inraw) tmpafdc(in=afdcownr); by ppid pppnum mnum;
/* If person did not have AFDC coverage but was AFDC owner   */
/* for someone else who did have AFDC coverage, change AFDC. */
 if inraw;
 if afdc ne 1 and afdcownr then
    do; afdc=1; chngafdc=1; end;
run;
title "months with AFDC changed to =1 (owner of coverage for someone else) - wave &nwav ";
proc freq; tables chngafdc /missing;
run;

data tst;
 set tmp&nwav;
 if chngafdc eq 1;
 prsnchng=put(ppid,z12.) || put(pppnum,z4.);
 keep prsnchng chngafdc;
run;
proc sort nodupkey; by prsnchng;
run;
title "number of people with AFDC changed to =1 - wave &nwav ";
proc freq; tables chngafdc;
run;


/* Add and/or Recode variables read in from the SIPP data. */
data tmp&nwav;
 set tmp&nwav;

/* Recode variables that have -1="not in universe" 1="yes      */
/* 2="no", to 0="not in universe OR no" to match older data.   */
 array tmp1{6} WDISAB WDISABP SSIREC AFDCREC GAREC OTHWREC;
 do tmp=1 to 6;
   if tmp1(tmp) in (-1,2) then tmp1(tmp)=0;
 end;
/* Recode variables that have -1="not in universe" but other   */
/* codes with specific values to 0="not in universe".          */
 array tmp2{5} ESR JOBID1 JOBID2 BUSID1 BUSID2 ;
 do tmp=1 to 5;
   if tmp2(tmp) eq -1 then tmp2(tmp)=0;
 end;

/* Set two race/ethnicity variables */
 black=(race = 2);
/* Pre-1996, "hispanic" was defined as 14 (Mexican-American),    */
/*    15 (Chicano), 16 (Mexican), 17 (Puerto Rican), 18 (Cuban), */
/*    19 (Central or South American Spanish), 20 (Other Spanish).*/
/* In the 1996 data, "hispanic" is defined as 20 (Mexican),      */
/*    21 (Mexican-American), 22 (Chicano), 23 (Puerto Rican),    */
/*    24 (Cuban), 25 (Central American), 26 (South American),    */
/*    27 (Dominican Republic), 28 (Other Hispanic).              */
 hispanic=(20 <= ethncty <= 28);

/* Recode education values to match HIGRAD in pre-1996 data */
/*
1996 EEDUCATE value                                max. schooling  HIGRAD
---------------------------------------------      --------------  ------
31 Less than 1st grade                               lt grade 1     0
                                                      grade  1      1
                                                      grade  2      2
                                                      grade  3      3
32 1st, 2nd, 3rd or 4th grade                         grade  4      4
                                                      grade  5      5
33 5th or 6th grade                                   grade  6      6
                                                      grade  7      7
34 7th or 8th grade                                   grade  8      8
35 9th grade                                          grade  9      9
36 10th grade                                         grade 10     10
37 11th grade                                         grade 11     11
38 12th grade                                         grade 12     12
39 High school diploma or GED                         grade 12     12
40 Some college but no degree                         grade 12     13
41 voc, tech, trade or bus school beyond HS           grade 12     13
                                                   1 yr college    13
42 Associate Degree (Occupational/vocational)      2 yrs college   14
43 Associate Degree (Academic program)             2 yrs college   14
                                                   3 yrs college   15
44 Bachelors degree (BA, AB, BS)                   4 yrs college   16
                                                   5 yrs college   17
45 Master's degree (MA, MS, MEng, MSW, MBA)        6 yrs college   18
46 Professional School Degree (MD,DDS,DVM,LLB,JD)  6 yrs college   18
47 Doctorate degree (PhD, EdD)                     6 yrs college   18
*/
 length hgrad enroll 3;
 if EEDUCATE eq 31 then HGRAD=0;
  else if EEDUCATE eq 32 then HGRAD=4;
  else if EEDUCATE eq 33 then HGRAD=6;
  else if EEDUCATE eq 34 then HGRAD=8;
  else if EEDUCATE eq 35 then HGRAD=9;
  else if EEDUCATE eq 36 then HGRAD=10;
  else if EEDUCATE eq 37 then HGRAD=11;
  else if EEDUCATE in(38,39) then HGRAD=12;
  else if EEDUCATE in(40,41) then HGRAD=13;
  else if EEDUCATE in(42,43) then HGRAD=14;
  else if EEDUCATE eq 44 then HGRAD=16;
  else if EEDUCATE in(45,46,47) then HGRAD=18;
  else HGRAD=.;

/* Cross-check if person is enrolled in school */
  if EENRLM eq 1 then ENROLL=1;
   else ENROLL=0;
/* Note that EENLEVEL has one value per wave (not month) */
  if EENLEVEL lt 1 then ENROLL=0; 
  drop EENRLM EENLEVEL EEDUCATE;

/* Set values for Health Insurance coverage.                */
/* EHIOWNER & ENONHH specify who owned the health insurance */
/* plan (both are defined once per wave).                   */
 if EHIOWNER in (1,3) then HIOWN=1; else HIOWN=0;
 if EHIOWNER in (2,3) then HIOTH=1; else HIOTH=0;
 if ENONHH eq 1 then HIOTH=1;
/* EHEMPLY defines the source of health insurance coverage: */
/* 1=employer, 2=FormerEmployer, 3=Union (once per wave).   */
 if EHEMPLY in (1,2,3) then HIEMP=1; else HIEMP=0;
/* EHIMTH defines coverage in month.                        */
/* If no insurance this month, zero variables.              */
 if EHIMTH ne 1 then do; HIOWN=0; HIOTH=0; HIEMP=0; end;

/* Recode some variables */
 array fix_0_2{6} mcare mcaid afdc champ foodst gasst;
 do tmp=1 to 6;
  if fix_0_2(tmp) in(-1,0) then fix_0_2(tmp)=.;
  if fix_0_2(tmp) eq 2 then fix_0_2(tmp)=0;
 end;
 drop tmp;

/* Compute related variables */
 mnotafdc=(mcaid=1 and afdc=0);
 priv=(hiown=1 or hioth=1 or hiemp=1 or champ=1);
 if esr in (1,2,3,4,5) then empl=1;
  else if esr ne . then empl=0;
  else empl=.;
 if esr eq 8 then nilf=1;
  else if esr ne . then nilf=0;
  else nilf=.;

/* Sum three of the welfare income amounts */
 allwel=persafdc+genasst+othwelf;

/* Create SUNIQUE just as Lara does */
 syear='96';
 id1='000000000000';
 id1=ppid;
 id3='0000';
 id3=pppnum;
 sippid=id1||id3;
 puid=sippid;
 sunique=syear||id1||id3;

/* Drop the variables not needed for final dataset */
 drop syear id1 id3 sippid puid
   ehemply ehimth ehiowner enonhh 
   rcuown20 rcutyp03 rcutyp04 rcutyp24 
   ssirecch ssiamtch ssiimpch ssirecst
   ssircimp ssiamtst siiimpst ;
run;

/* Append each of the 12 temporary datasets to the big file. */
proc datasets nolist;
 append out=more96  new=tmp&nwav;  delete tmp&nwav;
run;

/* End of loop through one wave of 1996 SIPP data */
%end;

/* End of macro */
%mend readraw;

/* Invoke the macro to loop through all 12 raw data files. */
%readraw;

/* Get rid of some temporary files */
proc datasets nolist; delete tmpafdc tst;
run;

/* Sort in order by PPID, PPPNUM, and reference month  */
/* (PPENTRY not needed in 1996 SIPP)                   */
proc sort data=more96; by ppid pppnum mnum;
run;

/* Merge in PNLWGT (Panel Weight) & FNLWTyy (Weight-for-Year) */
/* which were made available on the Census Bureau web site as */
/* a seperate file dated 01/08/2003.  Adjustment factors are  */
/* applied as specified in the "SIPP 1996: Longitudinal Weight*/
/* Adjustment Factor User Note" posted 10/14/05.              */
data wght96;
 infile 'ctl_fer.dat';
 input
 @13 ppid     12. /* SSUID    */
 @25 pppnum    4. /* EPPPNUM  */
 @29 pnlwgt   10. /* LGTPNLWT */
 @39 fnlwt96  10. /* LGTCY1WT */
 @49 fnlwt97  10. /* LGTCY2WT */
 @59 fnlwt98  10. /* LGTCY3WT */
 @69 fnlwt99  10. /* LGTCY4WT */
;
/* Apply adjustment factor as specified by Census Bureau */
 fnlwt97=fnlwt97*0.9812;
 fnlwt98=fnlwt98*0.9839;
 fnlwt99=fnlwt99*0.9819;
run;
proc sort; by ppid pppnum;
run;
/* Find who was present in March of 1997 */
libname x '';
data mar97;
 set x.fam96(keep=ppid pppnum date);
 if date eq 9703;
 drop date;
run;
/* Reset 1997 CalendarYearWeight to zero for all persons who */
/* are not present in March 1997 (as per Census Bureau).     */
data wght96;
 merge wght96 mar97(in=m97); by ppid pppnum;
 if not m97 then fnlwt97=0;
run;
/* Add weights to other data */
data more96;
 merge more96(in=in96sipp) wght96; by ppid pppnum;
 if in96sipp;
run;
proc datasets nolist; delete wght96 mar97;
run;


/* Get totals for income and welfare data for each family */
/* which includes reference person, spouse of reference   */
/* person, all children of reference person regardless of */
/* age, and RELAT=14 children (i.e., other relative of    */
/* reference person who has RefPerson as their Guardian). */
/* First, get family definition information for desired people */
data famly;
 set family.fam96;
 if relat in(1,2,3,14);
 keep sunique mnum ppid hhaddi famid;
/* Add family definition to income&welfare variables */
data famly2;
 merge more96(in=inc_wel) famly(in=fam); by sunique mnum;
 if inc_wel and fam;
 keep mnum ppid hhaddi famid ppinc ppearn allwel persafdc afdc;
proc sort; by mnum ppid hhaddi famid;
proc summary data=famly2;
 by mnum ppid hhaddi famid;
 var ppinc ppearn allwel persafdc afdc;
 output out=faminc
   sum(ppinc)=faminc
   sum(ppearn)=famwsal
   sum(allwel)=famwelf
   sum(persafdc)=famafdc
   sum(afdc)=anyafdc;
run;
proc datasets; delete famly2;
/* Add new family income and welfare variables to dataset */
proc sort data=famly; by mnum ppid hhaddi famid;
data famly;
 merge famly faminc; by mnum ppid hhaddi famid;
 keep sunique mnum faminc famwsal famwelf famafdc anyafdc;
run;
proc sort data=famly; by sunique mnum;
data more96; 
 merge more96 famly; by sunique mnum;
 length mnum race ethncty wdisab wdisabp 
   esr jobid1 jobid2 busid1 busid2 
   mcare mcaid afdc foodst gasst champ hiown hioth hiemp
   chngafdc  black hispanic empl nilf mnotafdc priv
   ssirec ssiimp afdcrec afdcimp garec gaimp othwrec othwimp
   anyafdc  3 ;
/* For months when a person is not part of a "family" (that is, */
/* a set of people with RELAT=1,2,3,14), set their "family"     */
/* values to be their individual values for those variables.    */
 if faminc  eq . then faminc =ppinc;
 if famwsal eq . then famwsal=ppearn;
 if famwelf eq . then famwelf=allwel;
 if famafdc eq . then famafdc=persafdc;
 if anyafdc eq . then anyafdc=afdc;
run;
proc datasets; delete famly faminc;
run;
title 'Means of Income and ProgramParticipation variables';
proc means;
run;

proc contents;
run;

/* MERGE IN DATA FROM EXTERNAL FILES *********************************/

/* libname sipp  '~ldshore/disk2'; */
libname sipp 'c:\SIPP_programs\extra_vars';
libname disab '';
libname immig '';

/* Make list of people in Family file,               */
/* keeping only the variables needed for merges.     */
/* Note that the temorary dataset FAMGUYS is used as */
/* a start; the temporary dataset ADDVARS is created */
/* from it and has additional variables added to it  */
/* step-by-step as files are merged in.              */

data famguys;
 set family.fam96;
 length afdcsize 3;
/* single parent & kids */
 if famstr in(1,3) then afdcsize=numkids+ochrefgd+1; 
/* two parents & kids */
 if famstr eq 2 then afdcsize=numkids+ochrefgd+2;    
 if afdcsize gt 9 and afdcsize ne . then afdcsize=9;
 keep sunique age mnum date state afdcsize ppid hhaddi famid relat
       ubrthmn ubrthyr;
run;


/* ************************************************************* */
/* ************** merge in the min_une data ******************** */
/* ************************************************************* */
data addvars;
 set famguys;
 length year month 3;
 year=int(date/100);
 month=date-(year*100);
run;
proc datasets; delete famguys;
proc sort data=addvars; by state year month;
proc sort data=sipp.min_une3 out=min_une3; by state year month;
data addvars;
 merge addvars(in=infam) min_une3(in=inmin); by state year month;
 if       infam=1 and inmin=1 then merged=1; /*indiv. in id'd states*/
  else if infam=1 and inmin=0 then merged=2; /*indiv in non-id'd states*/
  else if infam=0 and inmin=1 then merged=3; /*data for non-id'd states*/
  else if infam=0 and inmin=0 then merged=4; /*should not exist*/
  else                             merged=5; /*should not exist*/
run;
proc datasets; delete min_une3;
run;
title 'Min_unemp merge';
proc freq; tables merged;
run;
data addvars;
 set addvars;
 if merged in(1,2);
 drop merged;
run;

/* ************************************************************* */
/* *************** merge in the ben10 data ********************* */
/* ************************************************************* */
proc sort data=addvars; by year state afdcsize;
proc sort data=sipp.ben10 out=ben10; by year state afdcsize;
data addvars;
 merge addvars(in=infam) ben10(in=inben); by year state afdcsize;
 if       infam=1 and inben=1 then merged=1; /*indiv. in id'd states*/
  else if infam=1 and inben=0 then merged=2; /*indiv in non-id'd states*/
  else if infam=0 and inben=1 then merged=3; /*data for non-id'd states/yrs*/
  else if infam=0 and inben=0 then merged=4; /*should not exist*/
  else                             merged=5; /*should not exist*/
run;
proc datasets; delete ben10;
run;
title 'Ben10 merge';
proc freq; tables merged;
run;
data addvars;
 set addvars;
 if merged in(1,2);
 mpov=povline/12;
 drop merged povline sname;
run;

***********************************************************;
***Section 6***********************************************;
***section to get eligibility criteria for children attached;
***********************************************************;

/* work with the kids to get the data in the shape needed */
data kids;
 set addvars
  (keep= sunique mnum ubrthyr ubrthmn date age cpstate hhaddi famid ppid
          relat minwage mpov);
/* Only keep children of reference person and other relative of */
/* reference person lt 19 and with reference person as guardian.*/
 if (relat eq 3 and age lt 19) or relat eq 14; 

/* make child's birthdate */
 brthyr=ubrthyr-1900;
 birthday=(brthyr*100)+ubrthmn;

/* Lara's code to compute POVKID,POVINF by state & date:*/
/* At bottom, creates AGEEL,MEDLEV,EL40M,EL30M per kid. */

/* TEMPORARY - CHANGE YEAR FORMAT !!!!!! */
 if year < 5000 then year=year+200000; /* Before 1950 seems safe */
%INCLUDE 'eligibility_criteria_for_children.sas';
 if year > 200000 then year=year-200000; /* CHAGE BACK !!!!!! */

/* Keep only the variables required for the final dataset */
keep  mnum age hhaddi famid ppid ageel medlev el40m el30m;
run;

/* Take Lara's AGEEL,MEDLEV,EL40M,EL30M per kid     */
/* and sum them by family to create                 */
/* AGEELIG,YOUNGLIM,OLDLIM,EL40MIN,EL30MIN          */
/* (i.e., kid's min/max/mean values, by family).    */
proc sort; by mnum ppid hhaddi famid;
proc summary data=kids;
 by mnum ppid hhaddi famid;
 var ageel medlev el40m el30m;
 output out=kiddat
   mean(ageel)=ageelig
   max(medlev)=younglim
   min(medlev)=oldlim
   mean(el40m)=el40min
   mean(el30m)=el30min;
run;
/* Add the Eligibility Criteria for Children to main dataset */
proc sort data=addvars; by mnum ppid hhaddi famid;
data addvars;
 length younglim oldlim 3;
 merge addvars kiddat;
 by mnum ppid hhaddi famid;
/* Determine whether mother's youngest child could qualify  */
/* for Medicaid if she worked 30 or 40 hours at the minimum */
 yelig40=(((minwage*160)/mpov)*100) < younglim;
 yelig30=(((minwage*120)/mpov)*100) < younglim;
/* Compute an additional variable */
 ylimamt=(younglim/100)*mpov;
 drop _type_ _freq_ ;
run;
title 'After Eligibility Criteria for Children merge';
proc contents;
run;
options nolabel;
proc means;
run;
options label;
proc datasets;
  delete kiddat kids;


/* ************************************************************* */
/* ******* merge in the Child and Adult Disability data ******** */
/* ************************************************************* */
/* Recode Disability info as requested by Lara in January 2004   */
/*                                                               */

/* Merge in Child Disability from TopicalModule5 */
proc sort data=addvars; by sunique;
data addvars;
 merge addvars(in=infam) family.kdisab96(keep=sunique kdisab); by sunique;
 if infam;
run;

/* Add variable DISKID - number of disabled kids in family  */
/* Note: DISKID=. if ANY kids in the family have KDISAB=.   */
/* because that means we don't know the answer to the       */
/* question about how many kids in the family are disabled. */
/* DISKID=0 implies there was at least one kid in the family*/
/* and that kid could have been identified as disabled, but */
/* there were no kids who WERE identified as disabled.      */
data kids;
 set addvars;
 if (relat eq 3 and age le 18) or relat eq 14; /* only keep kids */
 keep mnum ppid hhaddi famid kdisab;
run;
proc sort data=kids; by mnum ppid hhaddi famid;
proc summary data=kids;
 by mnum ppid hhaddi famid;
 var kdisab;
 output out=diskd 
   sum(kdisab)=diskid     /* get sum of disabled kids in family */
   nmiss(kdisab)=dismiss; /* get number of missing values of KDISAB */
run;
proc sort data=addvars; by mnum ppid hhaddi famid;
data addvars;
 merge addvars diskd; by mnum ppid hhaddi famid;
 length diskid 3;
/* Reset to missing if ANY kids in family have KDISAB=. */
 if dismiss gt 0 then diskid=.;  
 drop _freq_ _type_ dismiss;
run;
proc datasets; delete kids diskd;
title 'Number of Disabled Children in family';
proc freq; tables diskid;
run;

/* Merge in Adult Disability from TopicalModule5 */
data adis;
 set disab.adisab96(keep=sunique adisab sevlim);
proc sort; by sunique;
proc sort data=addvars; by sunique;
data addvars;
 merge addvars(in=infam) adis(in=indis); by sunique;
 if       infam=1 and indis=1 then merged=1; /*cases in both data sets*/
  else if infam=1 and indis=0 then merged=2; /*not interv. in the dis wave*/
  else if infam=0 and indis=1 then merged=3; /*ppmis not 1 in core data*/
  else if infam=0 and indis=0 then merged=4; /*should be none of these*/
  else                             merged=5; /*should be none of these*/
run;
proc datasets; delete adis;
run;
title 'Adult Disability Merge';
proc freq; tables merged;
run;

data addvars;
 set addvars;
 if merged in(1,2);
 drop merged;
run;

data chk;
 set addvars;
 if adisab eq 1 and age le 18;
title 'Adult Disability for people less than 19 years old';
proc freq; tables adisab*age /missing norow nocol nopercent;
run;

/* Create DISAB based on ADISAB and KDISAB and whether the */
/* person was around for TopicalModule5.                   */
/*                                                         */
/* Make list of people in Family file who were around for TM5 */
data tm5guys;
 set addvars;
/* Keep only those in data for wave5 (include interview month) */
 if mnum in(17,18,19,20,21);  
 keep sunique;
proc sort nodupkey; by sunique;
run;
data addvars;
 length kdisab adisab sevlim disab 3;
 merge addvars tm5guys(in=tm5); by sunique;
  disab=.;
/* Use all KDISAB values */
 if kdisab ne . then disab=kdisab;
/* If person is an adult and exists in wave 5 */
 if age ge 14 and tm5 then
/* Use ADISAB =1 */
   if adisab eq 1 then disab=1;
/* Use ADISAB =0 only if DISAB not already =1 from KDISAB */
    else if adisab eq 0 and disab ne 1 then disab=0;
run;
proc datasets; delete tm5guys;

/* Merge in Immigration data from TopicalModule2 */
data immig;
 length immig noncit 3;
 set immig.immig96;
 keep sunique immig noncit;
proc sort; by sunique;
run;
data addvars;
 merge addvars (in=infam) immig(in=inimm); by sunique;
 if       infam=1 and inimm=1 then merged=1; /*cases in both data*/
  else if infam=1 and inimm=0 then merged=2; /*not interv in imm. wave*/
  else if infam=0 and inimm=1 then merged=3; /*ppmis not 1 in core data*/
  else if infam=0 and inimm=0 then merged=4; /*should be none of these*/
  else                             merged=5; /*should be none of these*/
run;
proc datasets; delete immig;
run;
title 'Immigrant Merge';
proc freq; tables merged;
run;

/* Set variable labels for Disability and Immigration variables */
data addvars;
 set addvars;
 if merged in(1,2);
 drop merged;
 label adisab= '=1 if adult disability'
       sevlim= '=1 if adult disability is severe'
       kdisab= '=1 if child disability'
       disab=  '=1 if disability (adult or child)'
       immig=  '=1 if person born outside US'
       noncit= '=1 if person not a US citizen';
run;
title 'Disability and immigrant vars post-merge';
proc means; var disab adisab sevlim kdisab immig noncit;
run;
proc freq; tables adisab*kdisab*sevlim /missing norow nocol nopercent;
run;

/* ************************************************************* */
/* *********** merge in the "fixed" HIGRAD data **************** */
/* ************************************************************* */
/* NO NEED TO DO THIS IN 1996 SIPP - EDUCATION DATA ok AS IS     */

/* ************************************************************* */
/* *************** merge in the fmapv6 data ******************** */
/* ************************************************************* */
proc sort data=sipp.fmapv6 out=fmapv6; by state year;
proc sort data=addvars; by state year;
data addvars;
 merge addvars(in=infam) fmapv6(in=infmp); by state year;
 if       infam=1 and infmp=1 then merged=1; /*obs I want*/
  else if infam=1 and infmp=0 then merged=2; /*states not id'd*/
  else if infam=0 and infmp=1 then merged=3; /*states not in SIPP96*/
  else if infam=0 and infmp=0 then merged=4; /*should not exist*/
  else                             merged=5; /*should not exist*/
run;
proc datasets; delete fmapv6;
run;
title 'FMAP merge';
proc freq; tables merged;
           tables merged*state /norow nocol nopercent;
           tables merged*year /norow nocol nopercent;
run;
data addvars;
 set addvars;
 if merged in(1,2);
 drop merged abbrev erate year4;
run;

/* ************************************************************* */
/* *************** merge in the mkreal data ******************** */
/* ************************************************************* */
proc sort data=addvars; by date;
proc sort data=sipp.mkreal out=mkreal; by date;
data addvars;
 merge addvars(in=infam) mkreal(in=inreal); by date;
 if       infam=1 and inreal=1 then merged=1; /*obs I want*/
  else if infam=1 and inreal=0 then merged=2; /*too early, too late*/
  else if infam=0 and inreal=1 then merged=3; /*dates not in SIPP96*/
  else if infam=0 and inreal=0 then merged=4; /*should not exist*/
  else                              merged=5; /*should not exist*/
run;
proc datasets; delete mkreal;
run;
title 'Mkreal merge';
proc freq; tables merged;
           tables merged*date /norow nocol nopercent;
run;
proc sort data=addvars; by sunique mnum;
run;
data addvars;
 set addvars;
 if merged in(1,2);
 drop merged;
run;


title 'Means of basic data after several merges';
options nolabel;
proc means;
run;
options label;
proc contents;


/* Merge the additional variables with the SIPP data */

data all96;
 merge family.fam96 more96; by sunique mnum;
run;

data family.all96;
 merge all96 addvars; by sunique mnum;

/* Add other necessary variables */
 length femhead twopar othhead 3 abbrev $ 2;

 femhead=(famstr=1);
 twopar=(famstr=2);
 othhead=(famstr in(3,4,5,6) );

 if state not in(61,62,63,0) then abbrev=fipstate(state);
 
/* Compute additional AFDC variables */
 gross=1.85*mneed;
 if date>8409 and date<8910 then do;
	md618=160*(kidlt18-kidlt6);
	md06=160*kidlt6;
	md612=160*(kidlt12-kidlt6);
	md1218=160*(kidlt18-kidlt12);

	md06_1=md06;
	md618_1=md618;
	md612_1=md612;
	md1218_1=md1218;

	yw=mpay+105;
	yw_1=1.5*mpay+105;
 end;
 else if date ge 8910 and date<9610 then do;
	md618=175*(kidlt18-kidlt6);
	md06=175*kidlt6+25*kidlt2;
	md612=175*(kidlt12-kidlt6);
	md1218=175*(kidlt18-kidlt12);

	md06_1=1.5*md06;
	md618_1=1.5*md618;
	md612_1=1.5*md612;
	md1218_1=1.5*md1218;

	yw=mpay+120;
	yw_1=1.5*mpay+120;
 end;
 afdclev1=min(gross, yw_1 + md06_1);
 afdclev2=min(gross, yw + md06);

/* Comment: Note the following when using the AFDC variables:   */
/* To run regressions with the afdc variables entered           */
/* separately, need to include                                  */
/*	(yw OR yw_1)	and                                     */
/*	daycare vars	--either                                */
/*	  (md06 and md618)--after 1st 4 mos. OR                 */
/*	  (md06_1 and md618_1)--1st 4 mos. OR                   */
/*	  (md06 and md612 and md1218)--after 1st 4 mos. OR      */
/*	  (md06_1 and md612_1 and md1218_1)--1st 4 mos.         */

/* Labels for the AFDC variables                                */
label 	gross= 'Gross test, nominal $'
	md618= 'Max daycare kids 6-17'
	md06= 'Max daycare kids < 6'
	md612= 'Max daycare kids 6-11'
	md1218= 'Max daycare kids 12-17'
	md06_1= 'Max daycare kids <6 1st 4 mos.'
	md618_1= 'Max daycare kids 6-17 1st 4 mos'
	md612_1= 'Max daycare kids 6-11 1st 4 mos'
	md1218_1= 'Max daycare kids 12-17 1st 4 mos'
	yw= 'Pay std plus disregard'
	yw_1= 'Pay std plus disregard 1st 4 mos'
	afdclev1= 'Max inc 1st 4 mos assum. dc kids <6 only'
	afdclev2= 'Max inc all mos assum. dc kids <6 only'
;

/* Include SAS code for Welfare Waiver variables and their labels */

/* TEMPORARY - CHANGE YEAR FORMAT !!!!!! */
 if year < 5000 then year=year+200000; /* Before 1950 seems safe */
%INCLUDE 'welfare_waivers.sas';
 if year > 200000 then year=year-200000; /* CHAGE BACK !!!!!! */

/* Labels for all the other variables                          */
label	
	abbrev   = 'Two-character state abbreviation'
	afdc     = '=1 if on AFDC'
	afdcimp  = 'Monthly AFDC receipt imputed'
	afdcrec  = 'Received AFDC income this month'
	afdcsize = 'Fam size for AFDC determination'
	age      = 'Age in years'
	ageelig  = 'Fraction of kids w/expansion eligibility'
	allwel   = 'Total welfare income for indiv'
	anyafdc  = 'Number of fam members on AFDC'
	black    = 'Individual is black'
	busid1   = 'Id of first self-employment'
	busid2   = 'Id of second self-employment'
	champ    = '=1 if on CHAMPUS'
	chngafdc = 'AFDC changed: consistant with older data'
	chpres   = 'Children in family related to RefPerson'
	dadpres  = 'Dad present in family this month'
	date     = 'Date in YYMM format'
	diskid   = 'Number of disabled kids in family'
	ehicost  = 'Employer paid all or part health ins. cost'
	el30min  = 'Frac kids elig if mom works 30hrs at min'
	el40min  = 'Frac kids elig if mom works 40hrs at min'
	empgr    = 'Employment growth rate in state'
	empl     = '=1 if employed'
	enroll   = '=1 if enrolled in school, Eileen fixed'
	esr      = 'Employment status recode'
	ethncty  = 'Ethnic origin--not mut excl w/race'
	famafdc  = 'Family income from AFDC'
	famid    = 'Family id (w/mnum,ppid, hhaddi)'
	faminc   = 'Family income'
	famnum   = 'Number of subfamily'
	famrel   = 'Relationship in subfamily'
	famstr   = 'Family structure code, smom=1,twopar=2'
	famtyp   = 'Primary or subfamily or individual'
	famwelf  = 'Family welfare income'
	famwsal  = 'Family wage and salary income'
	femhead  = 'Female headed-family'
	fnlwt96  = 'Weight for 1996 year'
	fnlwt97  = 'Weight for 1997 year'
	fnlwt98  = 'Weight for 1998 year'
	fnlwt99  = 'Weight for 1999 year'
	foodst   = 'Recv food stamps 1=yes'
	gaimp    = 'Monthly GA receipt imputed'
	garec    = 'Received GA income this month'
	gasst    = 'Recv general asst 1=yes'
	genasst  = 'Amount of GA rec this month'
	hgrad    = 'Highest grade completed, Eileen fixed'
	hhaddi   = 'Current household address id'
	hiemp    = '=1 if HI through employer'
	hioth    = '=1 if HI in others name'
	hiown    = '=1 if HI in own name'
	hispanic = 'Individual has Hispanic ethnicity'
	intmon   = 'Interview month within wave, =1-4'
	jobid1   = 'Id of first job'
	jobid2   = 'Id of second job'
	kidlt12  = 'Number of kids lt 12 in family'
	kidlt18  = 'Number of kids lt 18 in family'
	kidlt2   = 'Number of kids lt 2 in family'
	kidlt6   = 'Number of kids lt 6 in family'
	mcaid    = '=1 if on Medicaid'
	mcare    = '=1 if on Medicare'
	minwage  = 'Current minimum wage'
	mkreal   = 'Multiplier to obtain real values'
	mneed    = 'State AFDC monthly need standard'
	mnotafdc = 'Recv Medicaid and not AFDC'
	mnum     = 'Month number'
	mompres  = 'Mom present in family this month'
	mpay     = 'State AFDC monthly payment standard'
	mpov     = 'Monthly poverty line, nominal $'
	ms       = 'married=1 2,wid=3,div=4,sep=5,nev mar=6'
	nilf     = '=1 if not in labor force'
	nmtot    = 'Number of months with PPMIS=1'
	npersns  = 'number of people in family this month'
	numkids  = 'Number of kids in family'
	nummths  = 'Number of months woman was smom'
	ochrefgd = '"Other" children w/RefPerson as Guardian'
	oldest   = 'Age of oldest child'
	oldlim   = 'Medicaid income limit for oldest child'
	othchldn = 'Other children in family (relat=4&5)'
	othhead  = 'Family has other head'
	othwelf  = 'Amount of oth welf rec this month'
	othwimp  = 'Monthly oth welf receipt imputed'
	othwrec  = 'Received oth welf. inc. this month'
	parguard = '=1 if PARID comes from Guardian info'
	parid    = 'SUNIQUE of mom, dad, or guardian'
	persafdc = 'Amount of AFDC rec this month'
	pndad    = 'Person Number of Dad'
	pnlwgt   = 'Weight in panel'
	pnmom    = 'Person Number of Mom'
	ppearn   = 'Person monthly earnings'
	ppentry  = 'Entry address id: part of unique id'
	ppid     = 'ID of hh: part of unique id'
	ppinc    = 'Person monthly income'
	pppnum   = 'Person no. w/in hh: part of unique id'
	priv     = 'Has private health ins'
	race     = 'white=1,black=2,native am.=3,asian=4'
	rate     = 'Fed Medicaid Assistance Pct to state'
	relat    = 'Family relat.,head=1,sp=2,kid=3,oth=4'
	rot      = 'Rotation group: 1-4'
	rrp      = 'Relationship to reference person'
	sex      = '1=male; 2=female'
	smom     = 'Single mom in this month'
	ssiamt   = 'Amount of SSI rec this month'
	ssiimp   = 'Monthly SSI receipt imputed'
	ssirec   = 'Received SSI income this month'
	twopar   = 'Two-parent family'
	ubrthmn  = 'Month of birth'
	ubrthyr  = 'Year of birth'
/*	wavflg   = 'Imputation data flag' */
	wdisab   = 'Work DISAB from main SIPP data'
	wdisabp  = 'Had work-preventing condition'
	wpfinwgt = 'Person weight - monthly value'
	yelig30  = 'Youngest elig if worked 30 hrs at min'
	yelig40  = 'Youngest elig if worked 40 hrs at min'
	ylimamt  = 'Nominal income limit for youngest'
	youngest = 'Age of youngest child'
	younglim = 'Medicaid income limit for youngest child'
;

run;

title 'Means of data for all people in the SIPP';
options nolabel;
proc means;
run;

title 'Means of women who are ever a SingleMother';
data tmp;
 set family.all96;
/* Get record if was SingleMother at least one month */
 if nummths gt 0; 
/* Get rid of moms in non-singularly identified states */
 if state in (61,62,63,0) then delete; 
run;
proc means;
run;

proc freq;
tables smom*relat;
run;

title 'Means of women who are a SingleMother in current month';
data tmp;
 set family.all96;
/* Get record if was SingleMother this month */
 if smom eq 1;
/* Get rid of moms in non-singularly identified states */
 if state in (61,62,63,0) then delete;
run;
proc means;
run;

proc freq;
tables smom*relat;
run;



***********************************************************;
*Note that all variables are NOT in real terms.  In order
*to do that, just multiply the variable in question by the 
*variable mkreal.  Note that mkreal is missing for 1985
*values, and only goes through 9612.
***********************************************************;

***********************************************************;
*Still need to attach the waiver data--for AFDC and for 
*Medicaid.  Also need to attach the SCHIP data if we want
*to be as complete as possible.
***********************************************************;

endsas;
