options linesize=80 nocenter;

/* Program to select data for children less than or equal to  */
/* 18 years of age with RELAT=3, 4, or 14 (child of reference */
/* person, other relative of reference person, or with the    */
/* reference person or spouse as their guardian).  It omits   */
/* records for people who live in non-singularly identified   */
/* states 61, 62,  63, and 0.                                 */

/* IMPORTANT NOTE: The "allYY" data computes all child-related
   variables using NUMKIDS+OCHREFGD (number of children less 
   than or equal to 18 years old who are identified as 'Child
   of Reference Person' PLUS number of children less than or 
   equal to 18 years old who are identified as 'Other Relative
   of Reference Person' and who have the Reference Person 
   specified as their Guardian).  It is important to use the 
   sum of these two variables whenever the "number of children
   in family" is needed, to be consistent with the way the other
   variables were computed.  The variables KIDLT18 KIDLT12 KIDLT6 
   KIDLT2 OLDEST YOUNGEST and all variables related to Eligibility 
   Criteria For Children all use the sum of these two values. 
   In the datasets created by this program, variable "NC" is
   this sum (i.e., NC=NUMKIDS+OCHREFGD = "Number of Children
   in Family").                                                 */  

/* This program treats 1-month differences in PRIV and MCAID
   as coding errors and changes them (sets flags if changed,
   to find how many changes were made, then deletes the flags). */

/* Using the data with 1-month differences removed, it defines
   Insurance Spells and Transitions                             */

/* A modified version of SUNIQUE is created: XUNIQUE contains no
   embedded blanks and is the same length and format for all
   SIPP years.  It is intended for use in dumping data to be
   processed by Fortran.                                        */

/* IMPORTANT NOTE: If a person has gaps in the months, all records
   following the first gap are deleted from the final data.     */

/* SAS transport file "kid86.xpt" is created.                   */


/* Variables created in this program:                             */
/*                                                                */
/* FIRST=1 indicates it's the first record for a person (missing) */
/*        otherwise)                                              */
/* LAST=1 indicates it's the last record for a person (missing    */
/*        otherwise)                                              */
/* INS indicates the type of insurance:                           */
/*    1 => No Insurance                                           */
/*    2 => Public Insurance (Medicaid)                            */
/*    3 => Any Private Insurance (could also have public)         */
/* TRTYPE indicates the type of insurance transition:             */
/*    1 => none to public                                         */
/*    2 => none to private                                        */
/*    3 => public to none                                         */
/*    4 => public to private                                      */
/*    5 => private to none                                        */
/*    6 => private to public                                      */
/* SPELL indicates what kind of insurance spell it is:            */
/*    1 => a No Insurance spell                                   */
/*    2 => a Public Insurance spell                               */
/*    3 => a Private Insurance spell                              */
/* SNUM indicates the spell number for this person:               */
/*    0 => an interrupted spell                                   */
/*    1 => 1st fresh spell of that kind for this person           */
/*    2 => 2nd fresh spell of that kind for this person           */
/*    3 .... and so on                                            */
/* DUR indicates the duration (in months) of this record in       */
/*     current spell                                              */
/* LAST_SP=1 indicates last record of a spell                     */
/* INSCENS indicates whether or not insurance spell is censored:  */
/*    0 => not censored                                           */
/*    1 => censored                                               */
/* WELFRINT is a Fresh or Interrupted Welfare Spell Code          */
/*    1 => Fresh Welfare Spell                                    */
/*    2 => Fresh Off Welfare Spell                                */
/*    3 => Interrupted Welfare Spell                              */
/*    4 => Interrupted Off Welfare Spell                          */
/* WELCENS in a Welfare Spell Censored Indicator                  */
/*    0 => Spell not censored                                     */
/*    1 => Spell is censored                                      */
/* WELMONTH is a Welfare Spell Type-Of-Month Code                 */
/*    1 => 1st month in spell                                     */
/*    5 => intermediate month in spell                            */
/*    7 => last month in spell, not censored                      */
/*    9 => last month in spell, censored                          */
/*   10 => 1-month spell, censored                                */
/*   11 => 1-month spell, not censored                            */
/* WELDUR indicates the Welfare Spell Duration                    */
/*   WELDUR   = {length of current welfare spell}                 */
/* WEL_INS indicates type of insurance after end of welfare spell */
/*    1 => ended a spell on welfare the previous month and has    */
/*         no insurance this month                                */
/*    2 => ended a spell on welfare the previous month and has    */
/*         public insurance this month                            */
/*    3 => ended a spell on welfare the previous month and has    */
/*         private insurance this month                           */
/*    4 => ended a spell off welfare the previous month and has   */
/*         no insurance this month                                */
/*    5 => ended a spell off welfare the previous month and has   */
/*         public insurance this month                            */
/*    6 => ended a spell off welfare the previous month and has   */
/*         private insurance this month                           */
/*    0 => otherwise                                              */


%let SIPPYEAR=90;  
%let NM=32;                /* Number of Months in this SIPP file */

libname sipp 'c:\SIPP_programs\dec04';

data kids;
 set sipp.all&SIPPYEAR;
/* Only keep variables needed for Insurance & Welfare Spells */
 keep sunique mnum priv mcaid afdc;
/* Delete all records from non-singularly identified states */
 if state in (61,62,63,0) then delete; 
/* Delete all records for people older than 18 */
 if age gt 18 or relat not in(3,4,14) then delete;
run;
proc sort; by sunique mnum;
run;

/* Remove 1-month differences in PRIV, MCAID, AFDC */
data kids;
 set kids; by sunique;

/* Temporary arrays to hold data for a person */
 array mnm{&NM}   _temporary_;
 array prv{&NM}   _temporary_;
 array mcd{&NM}   _temporary_;
 array afd{&NM}   _temporary_;

/* Initialize arrays for first record of person */
 if first.sunique then 
   do i=1 to &NM;
     mnm(i)=.;
     prv(i)=.;
     mcd(i)=.;
     afd(i)=.;
   end;

/* Fill in the array with values for this person */
 mnm(mnum)=1;
 prv(mnum)=priv;
 mcd(mnum)=mcaid;
 afd(mnum)=afdc;

/* When come to last record for person, do special things */
 if last.sunique then
 do;

/* Initialize "change" flags */
 array pflg{&NM} _temporary_;
 array mflg{&NM} _temporary_;
 array wflg{&NM} _temporary_;
 do i=1 to &NM;
  pflg(i)=0;
  mflg(i)=0;
  wflg(i)=0;
 end;

/* If the last, current, and next records are all for the same
   person, with no gaps in the month numbers, then check for
   1-month differences in PRIV,MCAID,AFDC and change them. */
 do i=2 to &NM-1;
  if mnm(i-1) eq 1 and mnm(i) eq 1 and mnm(i+1) eq 1 then
   do;
    if pflg(i-1) eq 0 /* no change was made to original PRIV value */
     then
      do;
      if prv(i-1) eq 1 and prv(i+1) eq 1 and prv(i) eq 0
       then do; prv(i)=1; pflg(i)=1; end;
      if prv(i-1) eq 0 and prv(i+1) eq 0 and prv(i) eq 1
       then do; prv(i)=0; pflg(i)=1; end;
      end;
    if mflg(i-1) eq 0 /* no change was made to original MCAID value */
     then
      do;
      if mcd(i-1) eq 1 and mcd(i+1) eq 1 and mcd(i) eq 0
       then do; mcd(i)=1; mflg(i)=1; end;
      if mcd(i-1) eq 0 and mcd(i+1) eq 0 and mcd(i) eq 1
       then do; mcd(i)=0; mflg(i)=1; end;
      end;
    if wflg(i-1) eq 0 /* no change was made to original AFDC value */
     then
      do;
      if afd(i-1) eq 1 and afd(i+1) eq 1 and afd(i) eq 0
       then do; afd(i)=1; wflg(i)=1; end;
      if afd(i-1) eq 0 and afd(i+1) eq 0 and afd(i) eq 1
       then do; afd(i)=0; wflg(i)=1; end;
      end;
   end;
 end;

/* Output the records for this person, with any changes included */
 keep sunique mnum priv mcaid P_FLAG M_FLAG afdc W_FLAG;
 do mnum=1 to &NM;
  PRIV=prv(mnum);
  MCAID=mcd(mnum);
  P_FLAG=pflg(mnum);
  M_FLAG=mflg(mnum);
  AFDC=afd(mnum);
  W_FLAG=wflg(mnum);
  if mnm(mnum) eq 1 then output;
 end;

 end;   /* end of "if last.sunique" block */
run;

/* Check the 1-month differences that were changed */
title 'Means of variables in the No-1-Month-Differences dataset';
proc means;
run;

title 'How many records were changed by removing 1-month differences?';
proc freq; tables P_FLAG M_FLAG W_FLAG;
run;

/*
title 'sample: first 2000 records in kids data';
options obs=2000;
proc print noobs;
run;
options obs=99999999;
*/

/* Define insurance spells and transitions */
data kids;  /* Data now has 1-month differences removed */
 length mnum priv mcaid 
        FIRST LAST INS TRTYPE SPELL SNUM DUR LAST_SP INSCENS 3;
 set kids; by sunique;

/* Temporary arrays to hold data for a person */
 array mnm{&NM}   _temporary_;
 array prv{&NM}   _temporary_;
 array mcd{&NM}   _temporary_;
 array afd{&NM}   _temporary_;

/* Do special things for the first record of a person */
 if first.sunique then 
  do;
/* Save month number of first month for this person */
/* and set spell counters.                          */
  retain firstm lastm numnone numpub numpriv;
  firstm=mnum;
  numnone=0; numpub=0; numpriv=0;
/* Initialize arrays for first record of person */
   do i=1 to &NM;
     mnm(i)=.;
     prv(i)=.;
     mcd(i)=.;
     afd(i)=.;
   end;
  end;

/* Fill in the array with values for this person */
 mnm(mnum)=1;
 prv(mnum)=priv;
 mcd(mnum)=mcaid;
 afd(mnum)=afdc;

/* When come to last record for person, do special things */
 if last.sunique then
 do;
/* Initialize arrays which will contain new insurance variables */
 array xsp{&NM} _temporary_;
 array xls{&NM} _temporary_;
 array xsn{&NM} _temporary_;
 array xdu{&NM} _temporary_;
 array xl2{&NM} _temporary_;
 array xin{&NM} _temporary_;
 array xtr{&NM} _temporary_;
 array xcn{&NM} _temporary_;
 do i=1 to &NM;
  xsp(i)=0; xls(i)=.; xsn(i)=.;
  xdu(i)=0; xl2(i)=.; xtr(i)=0;
 end;

/* Set values of INS based on MCAID and PRIV:         */
/*  1 = No Insurance; 2 = Public Ins; 3 = Private Ins */ 
 do i=1 to &NM;
  if mcd(i) eq 0 and prv(i) eq 0 then xin(i)=1;
  if mcd(i) eq 1 and prv(i) eq 0 then xin(i)=2;
  if                 prv(i) eq 1 then xin(i)=3; 
 end;
/* Set values of TRTYPE based on changes in INS       */
 do i=2 to &NM;
  if xin(i-1) eq 1 and xin(i) eq 2 then xtr(i)=1; /* none2pub  */
  if xin(i-1) eq 1 and xin(i) eq 3 then xtr(i)=2; /* none2priv */
  if xin(i-1) eq 2 and xin(i) eq 1 then xtr(i)=3; /* pub2none  */
  if xin(i-1) eq 2 and xin(i) eq 3 then xtr(i)=4; /* pub2priv  */
  if xin(i-1) eq 3 and xin(i) eq 1 then xtr(i)=5; /* priv2none */
  if xin(i-1) eq 3 and xin(i) eq 2 then xtr(i)=6; /* priv2pub  */
  if xin(i-1) eq xin(i)            then xtr(i)=0; /* no change */
end;

/* Define insurance status in first month (which is included in spell) */
 xsp(firstm)=xin(firstm);
 xls(firstm)=.;
 xsn(firstm)=0;
 xdu(firstm)=1;
 xl2(firstm)=.;
 xtr(firstm)=0;
/* Check if only one record for this person */
 if firstm eq &NM or mnm(firstm+1) ne 1 then
  do; xls(firstm)=1; xl2(firstm)=1; end;

/* Loop through all their other months */
 do i=firstm+1 to &NM;
/* If a person has gaps in the months, all records   */ 
/* following the first gap are deleted from the data.*/
 if mnm(i) ne 1 then
   do;
   lastm=i-1; /* Set last record number as the one before gap */
   go to nomore1;
   end;

/* If no gap in months, define new variables */
/* SPELL: */
 if  (xin(i) eq 1 and xtr(i) eq 0) or
     (xtr(i) in(1,2))                 then xsp(i)=1;
 if  (xin(i) eq 2 and xtr(i) eq 0) or
     (xtr(i) in(3,4))                 then xsp(i)=2;
 if  (xin(i) eq 3 and xtr(i) eq 0) or
     (xtr(i) in(5,6))                 then xsp(i)=3;
/* Count number of spells of each type */
 if xsp(i) ne xsp(i-1) and (i-1) ne firstm then
   do;
   if xsp(i) eq 1 then numnone=numnone+1;
   if xsp(i) eq 2 then numpub =numpub+1;
   if xsp(i) eq 3 then numpriv=numpriv+1;
   end;
/* SNUM: */
 if xsp(i) eq 1 then xsn(i)=numnone;
 if xsp(i) eq 2 then xsn(i)=numpub;
 if xsp(i) eq 3 then xsn(i)=numpriv;
/* DUR: */
 if xsp(i) eq xsp(i-1) then xdu(i)=xdu(i-1)+1;
 else xdu(i)=1;
/* LAST_SP: */
 if xtr(i) ne 0 then xl2(i)=1;

 end; /* end of looping through all months */
 lastm=&NM;

/* No more months for this person */
nomore1:;
/* Mark last month of spell and last month of person */
/* LAST_SP: */
 xl2(lastm)=1;
/* LAST: */
 xls(lastm)=1;
/* Set censored flag for all the months */
/* No transition in last month => censored spell */
 if xtr(lastm) eq 0 then xcn(lastm)=1;
 if lastm ne firstm then
   do i=lastm-1 to firstm by -1;
    if xsp(i) eq xsp(lastm) and xsn(i) eq xsn(lastm) then
       xcn(i)=xcn(lastm);
    else xcn(i)=0;
   end;


/* Next, create variables for WELFARE/OFF-WELFARE group */

 length afdc WELFRINT WELCENS WELMONTH WELDUR WEL_INS 3;
/* Initialize arrays which will contain new variables */
 array wfrint{&NM} _temporary_;
 array wcens{&NM} _temporary_;
 array wmon{&NM} _temporary_;
 array wdur{&NM} _temporary_;
 array wins{&NM} _temporary_;
 do i=1 to &NM;
  wfrint(i)=.; wcens(i)=.; wmon(i)=.; wdur(i)=.; wins(i)=0;
 end;

/* Define type of spell in their first month */
 if afd(firstm) eq 1 then wfrint(firstm)=3;
 if afd(firstm) eq 0 then wfrint(firstm)=4;
 wmon(firstm)=1;
 wdur(firstm)=1;

/* Loop through all their other months */
 do i=firstm+1 to &NM;
/* If there is a gap in the months, don't use following data */
  if mnm(i) ne 1 then
    do;
    lastm=i-1;
    go to nomore2;
    end;
/* If no gap in months, set variables for spell */
  if afd(i) eq afd(i-1) then  /* continuation of spell */
    do;
    wfrint(i)=wfrint(i-1);  /* same "spell" code */
    wmon(i)=5;              /* intermediate month in spell */
    wdur(i)=wdur(i-1)+1;    /* add 1 to length of current spell */
    end;
  if afd(i) ne afd(i-1) then /* started a different type of spell */
    do;
   /* Mark end of previous spell */
    wmon(i-1)=7;            /* last month in previous spell (uncensored) */
   /* Set values for start of new spell */
    if afd(i) eq 1 then wfrint(i)=1; /* fresh welfare spell */
    if afd(i) eq 0 then wfrint(i)=2; /* fresh off welfare spell */
    wmon(i)=1;              /* first month in new spell */
    wdur(i)=1;              /* set length of this spell to 1 */
    end;

 end;  /* end of looping through all months */
 lastm=&NM; /* set last month for person, if no gaps earlier */
/* No more months for this person */
nomore2:;
/* Mark last month of last spell as censored */
 wmon(lastm)=9;

/* Add spell censored indicator to all the records */
/* and fix SpellType code for 1-month spells       */
 do i=lastm to firstm by -1;
  if wmon(i) eq 7 then wcens(i)=0;
  if wmon(i) eq 9 then wcens(i)=1;
  if wmon(i) not in(7,9) and i ne lastm then wcens(i)=wcens(i+1);
  if wmon(i) eq 7 and wdur(i) eq 1 
    then wmon(i)=11; /* 1-month spell, not censored */
  if wmon(i) eq 9 and wdur(i) eq 1
    then wmon(i)=10; /* 1-month spell, censored */
 end;

/* Add code to define the type of insurance in a month */
/* following the end of an uncensored welfare spell.   */
 wins(firstm)=0;
 do i=firstm+1 to lastm;
  wins(i)=0;
  if wmon(i-1) in(7,11) then /* Last month in uncensored spell */
   do;
    if wfrint(i-1) in(1,3) then /* Spell was On Welfare */
      wins(i)=xin(i);
    if wfrint(i-1) in(2,4) then /* Spell was Off Welfare */
      do;
      if xin(i) eq 1 then wins(i)=4;
      if xin(i) eq 2 then wins(i)=5;
      if xin(i) eq 3 then wins(i)=6;
      end;
   end;
 end;
  
/* Output the records for this person, with all the new variables */
 keep sunique mnum priv mcaid 
      first last ins trtype spell snum dur last_sp inscens
      afdc welfrint welcens welmonth weldur wel_ins;
 do mnum=firstm to lastm;
  if mnum eq firstm then FIRST=1; else FIRST=.;
  LAST    =xls(mnum);
  PRIV    =prv(mnum);
  MCAID   =mcd(mnum);
  INS     =xin(mnum);
  TRTYPE  =xtr(mnum);
  SPELL   =xsp(mnum);
  SNUM    =xsn(mnum);
  DUR     =xdu(mnum);
  LAST_SP =xl2(mnum);
  INSCENS =xcn(mnum);
  AFDC    =afd(mnum);
  WELFRINT=wfrint(mnum);
  WELCENS =wcens(mnum);
  WELMONTH=wmon(mnum);
  WELDUR  =wdur(mnum);
  WEL_INS =wins(mnum);

  if mnm(mnum) eq 1 then output;
 end;
 end;  /* End of .last code */
run;

data tmp;
 set kids;
 file "allnewkids&SIPPYEAR";
 put  sunique $18. +1 mnum 3. +1
  (priv mcaid ins trtype spell last snum dur last_sp inscens
    first afdc welfrint welcens welmonth weldur wel_ins)
  ( +1 2.) ;
run;

/* Merge Insurance Spell data into full data */
data kids&SIPPYEAR;
 merge kids(in=inkids)
       sipp.all&SIPPYEAR (drop=priv mcaid afdc);
       by sunique mnum;
 if inkids;
/* Create new version of SUNIQUE with no embedded blanks */
/* and big enough (18 characters) to hold 1992 SIPP IDs. */
 length xunique $ 18;
 syr=input(substr(sunique,1,2),2.);
 xunique=put(syr,z2.)||put(ppid,z9.)||put(ppentry,z3.)||put(pppnum,z4.);
 drop syr;
/* Add NC variable */
 nc=numkids+ochrefgd;
/* Define labels  */
label mnum    = 'Month number'
      first   = '1=first record of a person'
      last    = '1=last record of a person'
      priv    = '=1 if Private Ins. (no 1-month spells)'
      mcaid   = '=1 if on Medicaid (no 1-month spells)'
      nc      = '# Children in family (NUMKIDS+OCHREFGD)'
      xunique = 'Person ID with no embedded blanks' 
      ins     = 'Insurance: 1=none; 2=public; 3=private'
      trtype  = 'type of transition'
      spell   = 'type of spell: 1=NoIns, 2=Pub, 3=Priv'
      snum    = 'spell number: 0=interrupted, 1=1st fresh'
      dur     = 'duration (month number in current spell)'
      last_sp = '1=last record of a spell'
      inscens = '1=censored insurance spell, 0=NotCensord'
      afdc    = '=1 if on AFDC'
      welfrint= 'Fresh/Interrupted welfare spell code'
      welcens = 'Welfare spell Censored indicator'
      welmonth= 'Welfare spell type-of-month code' 
      weldur  = 'Welfare spell Duration'
      wel_ins = 'ins after welfare trans in previous mnth' ;
run;

/* Save as a transport file */
libname kidout xport "kids&SIPPYEAR..xpt";
proc copy in=WORK out=kidout; select kids&SIPPYEAR;
run;

options nolabel;
title 'Means for all data in Transport File';
proc means;
run;

/* Look in LOG to see how many PEOPLE are in transport file */
proc sort nodupkey data=kidout.kids&SIPPYEAR (keep=sunique)
   out=tmp; by sunique;
run;

